""" Auto-generated file. DO NOT MODIFY """
import enum
from typing import Dict, Any, Union, List, Iterable
from ansys.meshing.prime.internals.comm_manager import CommunicationManager
from ansys.meshing.prime.internals import utils
from ansys.meshing.prime.autogen.coreobject import *
import numpy as np

from ansys.meshing.prime.params.primestructs import *

class VolumeFillType(enum.IntEnum):
    """Types of volume fill options.
    """
    TET = 0
    """Volume fill using tetrahedral cells."""
    POLY = 1
    """Volume fill using polyhedral cells."""
    HEXCORETET = 2
    """Volume fill using hexahedral cells in the core and tetrahedral cells near the boundary."""
    HEXCOREPOLY = 3
    """Volume fill using hexahedral cells in the core and polyhedral cells near the boundary."""

class HexCoreCellElementType(enum.IntEnum):
    """Cell element type of hex-shaped cells.
    """
    ALLPOLY = 0
    """Generates poly type cells."""
    ALLHEX = 1
    """Generates hex type cells."""
    HEXANDPOLY = 2
    """Generates a mix of poly type and hex type cells."""

class HexCoreTransitionLayerType(enum.IntEnum):
    """Handle size transition of hex cells.
    """
    HANGINGNODES = 0
    """Allow different size hexes to be neighbors."""
    DELETESMALL = 1
    """Remove neighboring hexes of different size by deleting a layer of smaller cells. The voids are filled with tetrahedral or polyhedral cells depending on volumeFillType in AutoMeshParams structure."""
    DELETELARGE = 2
    """Remove neighboring hexes of different size by deleting a layer of larger cells. The voids are filled with tetrahedral or polyhedral cells depending on volumeFillType in AutoMeshParams structure."""
    DELETEBOTH = 3
    """Remove neighboring hexes of different size by deleting both smaller and larger cell layers. The voids are filled with tetrahedral or polyhedral cells depending on volumeFillType in AutoMeshParams structure."""

class AutoMeshResults(CoreObject):
    """Results of volume meshing.
    """
    _default_params = {}

    def __initialize(
            self,
            error_code: ErrorCode,
            warning_codes: List[WarningCode],
            error_locations: Iterable[float]):
        self._error_code = ErrorCode(error_code)
        self._warning_codes = warning_codes
        self._error_locations = error_locations if isinstance(error_locations, np.ndarray) else np.array(error_locations, dtype=np.double) if error_locations is not None else None

    def __init__(
            self,
            model: CommunicationManager=None,
            error_code: ErrorCode = None,
            warning_codes: List[WarningCode] = None,
            error_locations: Iterable[float] = None,
            json_data : dict = None,
             **kwargs):
        """Initializes the AutoMeshResults.

        Parameters
        ----------
        model: Model
            Model to create a AutoMeshResults object with default parameters.
        error_code: ErrorCode, optional
            Provides error message when automesh fails.
        warning_codes: List[WarningCode], optional
            Warning codes associated with the operation.
        error_locations: Iterable[float], optional
            Error location coordinates returned when automesh fails.
        json_data: dict, optional
            JSON dictionary to create a AutoMeshResults object with provided parameters.

        Examples
        --------
        >>> auto_mesh_results = prime.AutoMeshResults(model = model)
        """
        if json_data:
            self.__initialize(
                ErrorCode(json_data["errorCode"] if "errorCode" in json_data else None),
                [WarningCode(data) for data in json_data["warningCodes"]] if "warningCodes" in json_data else None,
                json_data["errorLocations"] if "errorLocations" in json_data else None)
        else:
            all_field_specified = all(arg is not None for arg in [error_code, warning_codes, error_locations])
            if all_field_specified:
                self.__initialize(
                    error_code,
                    warning_codes,
                    error_locations)
            else:
                if model is None:
                    raise ValueError("Invalid assignment. Either pass model or specify all properties")
                else:
                    param_json = model._communicator.initialize_params(model, "AutoMeshResults")
                    json_data = param_json["AutoMeshResults"] if "AutoMeshResults" in param_json else {}
                    self.__initialize(
                        error_code if error_code is not None else ( AutoMeshResults._default_params["error_code"] if "error_code" in AutoMeshResults._default_params else ErrorCode(json_data["errorCode"] if "errorCode" in json_data else None)),
                        warning_codes if warning_codes is not None else ( AutoMeshResults._default_params["warning_codes"] if "warning_codes" in AutoMeshResults._default_params else [WarningCode(data) for data in (json_data["warningCodes"] if "warningCodes" in json_data else None)]),
                        error_locations if error_locations is not None else ( AutoMeshResults._default_params["error_locations"] if "error_locations" in AutoMeshResults._default_params else (json_data["errorLocations"] if "errorLocations" in json_data else None)))
        self._custom_params = kwargs
        if model is not None:
            [ model._logger.warning(f'Unsupported argument : {key}') for key in kwargs ]
        [setattr(type(self), key, property(lambda self, key = key:  self._custom_params[key] if key in self._custom_params else None,
        lambda self, value, key = key : self._custom_params.update({ key: value }))) for key in kwargs]
        self._freeze()

    @staticmethod
    def set_default(
            error_code: ErrorCode = None,
            warning_codes: List[WarningCode] = None,
            error_locations: Iterable[float] = None):
        """Set the default values of AutoMeshResults.

        Parameters
        ----------
        error_code: ErrorCode, optional
            Provides error message when automesh fails.
        warning_codes: List[WarningCode], optional
            Warning codes associated with the operation.
        error_locations: Iterable[float], optional
            Error location coordinates returned when automesh fails.
        """
        args = locals()
        [AutoMeshResults._default_params.update({ key: value }) for key, value in args.items() if value is not None]

    @staticmethod
    def print_default():
        """Print the default values of AutoMeshResults.

        Examples
        --------
        >>> AutoMeshResults.print_default()
        """
        message = ""
        message += ''.join(str(key) + ' : ' + str(value) + '\n' for key, value in AutoMeshResults._default_params.items())
        print(message)

    def _jsonify(self) -> Dict[str, Any]:
        json_data = {}
        if self._error_code is not None:
            json_data["errorCode"] = self._error_code
        if self._warning_codes is not None:
            json_data["warningCodes"] = [data for data in self._warning_codes]
        if self._error_locations is not None:
            json_data["errorLocations"] = self._error_locations
        [ json_data.update({ utils.to_camel_case(key) : value }) for key, value in self._custom_params.items()]
        return json_data

    def __str__(self) -> str:
        message = "error_code :  %s\nwarning_codes :  %s\nerror_locations :  %s" % (self._error_code, '[' + ''.join('\n' + str(data) for data in self._warning_codes) + ']', self._error_locations)
        message += ''.join('\n' + str(key) + ' : ' + str(value) for key, value in self._custom_params.items())
        return message

    @property
    def error_code(self) -> ErrorCode:
        """Provides error message when automesh fails.
        """
        return self._error_code

    @error_code.setter
    def error_code(self, value: ErrorCode):
        self._error_code = value

    @property
    def warning_codes(self) -> List[WarningCode]:
        """Warning codes associated with the operation.
        """
        return self._warning_codes

    @warning_codes.setter
    def warning_codes(self, value: List[WarningCode]):
        self._warning_codes = value

    @property
    def error_locations(self) -> Iterable[float]:
        """Error location coordinates returned when automesh fails.
        """
        return self._error_locations

    @error_locations.setter
    def error_locations(self, value: Iterable[float]):
        self._error_locations = value

class PrismStairStep(CoreObject):
    """Parameters to control prism stairsteping.
    """
    _default_params = {}

    def __initialize(
            self,
            check_proximity: bool,
            gap_factor_scale: float):
        self._check_proximity = check_proximity
        self._gap_factor_scale = gap_factor_scale

    def __init__(
            self,
            model: CommunicationManager=None,
            check_proximity: bool = None,
            gap_factor_scale: float = None,
            json_data : dict = None,
             **kwargs):
        """Initializes the PrismStairStep.

        Parameters
        ----------
        model: Model
            Model to create a PrismStairStep object with default parameters.
        check_proximity: bool, optional
            Check whether to enable or disable stairstepping at prisms within proximity of boundary or prism cap.
        gap_factor_scale: float, optional
            Scale factor for prism proximity detection gap factor.
        json_data: dict, optional
            JSON dictionary to create a PrismStairStep object with provided parameters.

        Examples
        --------
        >>> prism_stair_step = prime.PrismStairStep(model = model)
        """
        if json_data:
            self.__initialize(
                json_data["checkProximity"] if "checkProximity" in json_data else None,
                json_data["gapFactorScale"] if "gapFactorScale" in json_data else None)
        else:
            all_field_specified = all(arg is not None for arg in [check_proximity, gap_factor_scale])
            if all_field_specified:
                self.__initialize(
                    check_proximity,
                    gap_factor_scale)
            else:
                if model is None:
                    raise ValueError("Invalid assignment. Either pass model or specify all properties")
                else:
                    param_json = model._communicator.initialize_params(model, "PrismStairStep")
                    json_data = param_json["PrismStairStep"] if "PrismStairStep" in param_json else {}
                    self.__initialize(
                        check_proximity if check_proximity is not None else ( PrismStairStep._default_params["check_proximity"] if "check_proximity" in PrismStairStep._default_params else (json_data["checkProximity"] if "checkProximity" in json_data else None)),
                        gap_factor_scale if gap_factor_scale is not None else ( PrismStairStep._default_params["gap_factor_scale"] if "gap_factor_scale" in PrismStairStep._default_params else (json_data["gapFactorScale"] if "gapFactorScale" in json_data else None)))
        self._custom_params = kwargs
        if model is not None:
            [ model._logger.warning(f'Unsupported argument : {key}') for key in kwargs ]
        [setattr(type(self), key, property(lambda self, key = key:  self._custom_params[key] if key in self._custom_params else None,
        lambda self, value, key = key : self._custom_params.update({ key: value }))) for key in kwargs]
        self._freeze()

    @staticmethod
    def set_default(
            check_proximity: bool = None,
            gap_factor_scale: float = None):
        """Set the default values of PrismStairStep.

        Parameters
        ----------
        check_proximity: bool, optional
            Check whether to enable or disable stairstepping at prisms within proximity of boundary or prism cap.
        gap_factor_scale: float, optional
            Scale factor for prism proximity detection gap factor.
        """
        args = locals()
        [PrismStairStep._default_params.update({ key: value }) for key, value in args.items() if value is not None]

    @staticmethod
    def print_default():
        """Print the default values of PrismStairStep.

        Examples
        --------
        >>> PrismStairStep.print_default()
        """
        message = ""
        message += ''.join(str(key) + ' : ' + str(value) + '\n' for key, value in PrismStairStep._default_params.items())
        print(message)

    def _jsonify(self) -> Dict[str, Any]:
        json_data = {}
        if self._check_proximity is not None:
            json_data["checkProximity"] = self._check_proximity
        if self._gap_factor_scale is not None:
            json_data["gapFactorScale"] = self._gap_factor_scale
        [ json_data.update({ utils.to_camel_case(key) : value }) for key, value in self._custom_params.items()]
        return json_data

    def __str__(self) -> str:
        message = "check_proximity :  %s\ngap_factor_scale :  %s" % (self._check_proximity, self._gap_factor_scale)
        message += ''.join('\n' + str(key) + ' : ' + str(value) for key, value in self._custom_params.items())
        return message

    @property
    def check_proximity(self) -> bool:
        """Check whether to enable or disable stairstepping at prisms within proximity of boundary or prism cap.
        """
        return self._check_proximity

    @check_proximity.setter
    def check_proximity(self, value: bool):
        self._check_proximity = value

    @property
    def gap_factor_scale(self) -> float:
        """Scale factor for prism proximity detection gap factor.
        """
        return self._gap_factor_scale

    @gap_factor_scale.setter
    def gap_factor_scale(self, value: float):
        self._gap_factor_scale = value

class PrismParams(CoreObject):
    """Parameters to control prism mesh generation.
    """
    _default_params = {}

    def __initialize(
            self,
            stair_step: PrismStairStep,
            no_imprint_zonelets: Iterable[int]):
        self._stair_step = stair_step
        self._no_imprint_zonelets = no_imprint_zonelets if isinstance(no_imprint_zonelets, np.ndarray) else np.array(no_imprint_zonelets, dtype=np.int32) if no_imprint_zonelets is not None else None

    def __init__(
            self,
            model: CommunicationManager=None,
            stair_step: PrismStairStep = None,
            no_imprint_zonelets: Iterable[int] = None,
            json_data : dict = None,
             **kwargs):
        """Initializes the PrismParams.

        Parameters
        ----------
        model: Model
            Model to create a PrismParams object with default parameters.
        stair_step: PrismStairStep, optional
            Prism stairstep parameters.
        no_imprint_zonelets: Iterable[int], optional
            Option to specify zonelets to skip prism imprint.
        json_data: dict, optional
            JSON dictionary to create a PrismParams object with provided parameters.

        Examples
        --------
        >>> prism_params = prime.PrismParams(model = model)
        """
        if json_data:
            self.__initialize(
                PrismStairStep(model = model, json_data = json_data["stairStep"] if "stairStep" in json_data else None),
                json_data["noImprintZonelets"] if "noImprintZonelets" in json_data else None)
        else:
            all_field_specified = all(arg is not None for arg in [stair_step, no_imprint_zonelets])
            if all_field_specified:
                self.__initialize(
                    stair_step,
                    no_imprint_zonelets)
            else:
                if model is None:
                    raise ValueError("Invalid assignment. Either pass model or specify all properties")
                else:
                    param_json = model._communicator.initialize_params(model, "PrismParams")
                    json_data = param_json["PrismParams"] if "PrismParams" in param_json else {}
                    self.__initialize(
                        stair_step if stair_step is not None else ( PrismParams._default_params["stair_step"] if "stair_step" in PrismParams._default_params else PrismStairStep(model = model, json_data = (json_data["stairStep"] if "stairStep" in json_data else None))),
                        no_imprint_zonelets if no_imprint_zonelets is not None else ( PrismParams._default_params["no_imprint_zonelets"] if "no_imprint_zonelets" in PrismParams._default_params else (json_data["noImprintZonelets"] if "noImprintZonelets" in json_data else None)))
        self._custom_params = kwargs
        if model is not None:
            [ model._logger.warning(f'Unsupported argument : {key}') for key in kwargs ]
        [setattr(type(self), key, property(lambda self, key = key:  self._custom_params[key] if key in self._custom_params else None,
        lambda self, value, key = key : self._custom_params.update({ key: value }))) for key in kwargs]
        self._freeze()

    @staticmethod
    def set_default(
            stair_step: PrismStairStep = None,
            no_imprint_zonelets: Iterable[int] = None):
        """Set the default values of PrismParams.

        Parameters
        ----------
        stair_step: PrismStairStep, optional
            Prism stairstep parameters.
        no_imprint_zonelets: Iterable[int], optional
            Option to specify zonelets to skip prism imprint.
        """
        args = locals()
        [PrismParams._default_params.update({ key: value }) for key, value in args.items() if value is not None]

    @staticmethod
    def print_default():
        """Print the default values of PrismParams.

        Examples
        --------
        >>> PrismParams.print_default()
        """
        message = ""
        message += ''.join(str(key) + ' : ' + str(value) + '\n' for key, value in PrismParams._default_params.items())
        print(message)

    def _jsonify(self) -> Dict[str, Any]:
        json_data = {}
        if self._stair_step is not None:
            json_data["stairStep"] = self._stair_step._jsonify()
        if self._no_imprint_zonelets is not None:
            json_data["noImprintZonelets"] = self._no_imprint_zonelets
        [ json_data.update({ utils.to_camel_case(key) : value }) for key, value in self._custom_params.items()]
        return json_data

    def __str__(self) -> str:
        message = "stair_step :  %s\nno_imprint_zonelets :  %s" % ('{ ' + str(self._stair_step) + ' }', self._no_imprint_zonelets)
        message += ''.join('\n' + str(key) + ' : ' + str(value) for key, value in self._custom_params.items())
        return message

    @property
    def stair_step(self) -> PrismStairStep:
        """Prism stairstep parameters.
        """
        return self._stair_step

    @stair_step.setter
    def stair_step(self, value: PrismStairStep):
        self._stair_step = value

    @property
    def no_imprint_zonelets(self) -> Iterable[int]:
        """Option to specify zonelets to skip prism imprint.
        """
        return self._no_imprint_zonelets

    @no_imprint_zonelets.setter
    def no_imprint_zonelets(self, value: Iterable[int]):
        self._no_imprint_zonelets = value

class SurfaceMeshSizeScaling(CoreObject):
    """Settings related to scaling of surface mesh size for hexcore refinement.
    """
    _default_params = {}

    def __initialize(
            self,
            factor: float,
            size_range_min: float,
            size_range_max: float):
        self._factor = factor
        self._size_range_min = size_range_min
        self._size_range_max = size_range_max

    def __init__(
            self,
            model: CommunicationManager=None,
            factor: float = None,
            size_range_min: float = None,
            size_range_max: float = None,
            json_data : dict = None,
             **kwargs):
        """Initializes the SurfaceMeshSizeScaling.

        Parameters
        ----------
        model: Model
            Model to create a SurfaceMeshSizeScaling object with default parameters.
        factor: float, optional
            Value by which size should be multiplied when the size falls within a certain range. Applicable only when sizeFieldType is set to Geometric in AutoMeshParams structure.
        size_range_min: float, optional
            Minimum size required to apply scaling. Applicable only when sizeFieldType is set to Geometric in AutoMeshParams structure.
        size_range_max: float, optional
            Maximum size required to apply scaling. Applicable only when sizeFieldType is set to Geometric in AutoMeshParams structure.
        json_data: dict, optional
            JSON dictionary to create a SurfaceMeshSizeScaling object with provided parameters.

        Examples
        --------
        >>> surface_mesh_size_scaling = prime.SurfaceMeshSizeScaling(model = model)
        """
        if json_data:
            self.__initialize(
                json_data["factor"] if "factor" in json_data else None,
                json_data["sizeRangeMin"] if "sizeRangeMin" in json_data else None,
                json_data["sizeRangeMax"] if "sizeRangeMax" in json_data else None)
        else:
            all_field_specified = all(arg is not None for arg in [factor, size_range_min, size_range_max])
            if all_field_specified:
                self.__initialize(
                    factor,
                    size_range_min,
                    size_range_max)
            else:
                if model is None:
                    raise ValueError("Invalid assignment. Either pass model or specify all properties")
                else:
                    param_json = model._communicator.initialize_params(model, "SurfaceMeshSizeScaling")
                    json_data = param_json["SurfaceMeshSizeScaling"] if "SurfaceMeshSizeScaling" in param_json else {}
                    self.__initialize(
                        factor if factor is not None else ( SurfaceMeshSizeScaling._default_params["factor"] if "factor" in SurfaceMeshSizeScaling._default_params else (json_data["factor"] if "factor" in json_data else None)),
                        size_range_min if size_range_min is not None else ( SurfaceMeshSizeScaling._default_params["size_range_min"] if "size_range_min" in SurfaceMeshSizeScaling._default_params else (json_data["sizeRangeMin"] if "sizeRangeMin" in json_data else None)),
                        size_range_max if size_range_max is not None else ( SurfaceMeshSizeScaling._default_params["size_range_max"] if "size_range_max" in SurfaceMeshSizeScaling._default_params else (json_data["sizeRangeMax"] if "sizeRangeMax" in json_data else None)))
        self._custom_params = kwargs
        if model is not None:
            [ model._logger.warning(f'Unsupported argument : {key}') for key in kwargs ]
        [setattr(type(self), key, property(lambda self, key = key:  self._custom_params[key] if key in self._custom_params else None,
        lambda self, value, key = key : self._custom_params.update({ key: value }))) for key in kwargs]
        self._freeze()

    @staticmethod
    def set_default(
            factor: float = None,
            size_range_min: float = None,
            size_range_max: float = None):
        """Set the default values of SurfaceMeshSizeScaling.

        Parameters
        ----------
        factor: float, optional
            Value by which size should be multiplied when the size falls within a certain range. Applicable only when sizeFieldType is set to Geometric in AutoMeshParams structure.
        size_range_min: float, optional
            Minimum size required to apply scaling. Applicable only when sizeFieldType is set to Geometric in AutoMeshParams structure.
        size_range_max: float, optional
            Maximum size required to apply scaling. Applicable only when sizeFieldType is set to Geometric in AutoMeshParams structure.
        """
        args = locals()
        [SurfaceMeshSizeScaling._default_params.update({ key: value }) for key, value in args.items() if value is not None]

    @staticmethod
    def print_default():
        """Print the default values of SurfaceMeshSizeScaling.

        Examples
        --------
        >>> SurfaceMeshSizeScaling.print_default()
        """
        message = ""
        message += ''.join(str(key) + ' : ' + str(value) + '\n' for key, value in SurfaceMeshSizeScaling._default_params.items())
        print(message)

    def _jsonify(self) -> Dict[str, Any]:
        json_data = {}
        if self._factor is not None:
            json_data["factor"] = self._factor
        if self._size_range_min is not None:
            json_data["sizeRangeMin"] = self._size_range_min
        if self._size_range_max is not None:
            json_data["sizeRangeMax"] = self._size_range_max
        [ json_data.update({ utils.to_camel_case(key) : value }) for key, value in self._custom_params.items()]
        return json_data

    def __str__(self) -> str:
        message = "factor :  %s\nsize_range_min :  %s\nsize_range_max :  %s" % (self._factor, self._size_range_min, self._size_range_max)
        message += ''.join('\n' + str(key) + ' : ' + str(value) for key, value in self._custom_params.items())
        return message

    @property
    def factor(self) -> float:
        """Value by which size should be multiplied when the size falls within a certain range. Applicable only when sizeFieldType is set to Geometric in AutoMeshParams structure.
        """
        return self._factor

    @factor.setter
    def factor(self, value: float):
        self._factor = value

    @property
    def size_range_min(self) -> float:
        """Minimum size required to apply scaling. Applicable only when sizeFieldType is set to Geometric in AutoMeshParams structure.
        """
        return self._size_range_min

    @size_range_min.setter
    def size_range_min(self, value: float):
        self._size_range_min = value

    @property
    def size_range_max(self) -> float:
        """Maximum size required to apply scaling. Applicable only when sizeFieldType is set to Geometric in AutoMeshParams structure.
        """
        return self._size_range_max

    @size_range_max.setter
    def size_range_max(self, value: float):
        self._size_range_max = value

class HexCoreParams(CoreObject):
    """Parameters to control hexahedral mesh generation.
    """
    _default_params = {}

    def __initialize(
            self,
            transition_size_field_type: SizeFieldType,
            buffer_layers: int,
            rel_peel_layer_offset: float,
            transition_layer_type: HexCoreTransitionLayerType,
            cell_element_type: HexCoreCellElementType,
            surface_mesh_size_scaling: SurfaceMeshSizeScaling,
            enable_region_based_hexcore: bool):
        self._transition_size_field_type = SizeFieldType(transition_size_field_type)
        self._buffer_layers = buffer_layers
        self._rel_peel_layer_offset = rel_peel_layer_offset
        self._transition_layer_type = HexCoreTransitionLayerType(transition_layer_type)
        self._cell_element_type = HexCoreCellElementType(cell_element_type)
        self._surface_mesh_size_scaling = surface_mesh_size_scaling
        self._enable_region_based_hexcore = enable_region_based_hexcore

    def __init__(
            self,
            model: CommunicationManager=None,
            transition_size_field_type: SizeFieldType = None,
            buffer_layers: int = None,
            rel_peel_layer_offset: float = None,
            transition_layer_type: HexCoreTransitionLayerType = None,
            cell_element_type: HexCoreCellElementType = None,
            surface_mesh_size_scaling: SurfaceMeshSizeScaling = None,
            enable_region_based_hexcore: bool = None,
            json_data : dict = None,
             **kwargs):
        """Initializes the HexCoreParams.

        Parameters
        ----------
        model: Model
            Model to create a HexCoreParams object with default parameters.
        transition_size_field_type: SizeFieldType, optional
            Size field type to be used for transition volume (volume between hexcore and boundary).
        buffer_layers: int, optional
            Minimum number of cell layers of the same size before the cell size halves or doubles.
        rel_peel_layer_offset: float, optional
            Gap between hexahedral core and geometry surface relative to the surface mesh size.
        transition_layer_type: HexCoreTransitionLayerType, optional
            Handle size transition of hex cells.
        cell_element_type: HexCoreCellElementType, optional
            Cell element type of hex-shaped cells.
        surface_mesh_size_scaling: SurfaceMeshSizeScaling, optional
            Settings related to scaling of surface mesh size for hexcore refinement.
        enable_region_based_hexcore: bool, optional
            Checks whether to enable region based hexcore or not.
        json_data: dict, optional
            JSON dictionary to create a HexCoreParams object with provided parameters.

        Examples
        --------
        >>> hex_core_params = prime.HexCoreParams(model = model)
        """
        if json_data:
            self.__initialize(
                SizeFieldType(json_data["transitionSizeFieldType"] if "transitionSizeFieldType" in json_data else None),
                json_data["bufferLayers"] if "bufferLayers" in json_data else None,
                json_data["relPeelLayerOffset"] if "relPeelLayerOffset" in json_data else None,
                HexCoreTransitionLayerType(json_data["transitionLayerType"] if "transitionLayerType" in json_data else None),
                HexCoreCellElementType(json_data["cellElementType"] if "cellElementType" in json_data else None),
                SurfaceMeshSizeScaling(model = model, json_data = json_data["surfaceMeshSizeScaling"] if "surfaceMeshSizeScaling" in json_data else None),
                json_data["enableRegionBasedHexcore"] if "enableRegionBasedHexcore" in json_data else None)
        else:
            all_field_specified = all(arg is not None for arg in [transition_size_field_type, buffer_layers, rel_peel_layer_offset, transition_layer_type, cell_element_type, surface_mesh_size_scaling, enable_region_based_hexcore])
            if all_field_specified:
                self.__initialize(
                    transition_size_field_type,
                    buffer_layers,
                    rel_peel_layer_offset,
                    transition_layer_type,
                    cell_element_type,
                    surface_mesh_size_scaling,
                    enable_region_based_hexcore)
            else:
                if model is None:
                    raise ValueError("Invalid assignment. Either pass model or specify all properties")
                else:
                    param_json = model._communicator.initialize_params(model, "HexCoreParams")
                    json_data = param_json["HexCoreParams"] if "HexCoreParams" in param_json else {}
                    self.__initialize(
                        transition_size_field_type if transition_size_field_type is not None else ( HexCoreParams._default_params["transition_size_field_type"] if "transition_size_field_type" in HexCoreParams._default_params else SizeFieldType(json_data["transitionSizeFieldType"] if "transitionSizeFieldType" in json_data else None)),
                        buffer_layers if buffer_layers is not None else ( HexCoreParams._default_params["buffer_layers"] if "buffer_layers" in HexCoreParams._default_params else (json_data["bufferLayers"] if "bufferLayers" in json_data else None)),
                        rel_peel_layer_offset if rel_peel_layer_offset is not None else ( HexCoreParams._default_params["rel_peel_layer_offset"] if "rel_peel_layer_offset" in HexCoreParams._default_params else (json_data["relPeelLayerOffset"] if "relPeelLayerOffset" in json_data else None)),
                        transition_layer_type if transition_layer_type is not None else ( HexCoreParams._default_params["transition_layer_type"] if "transition_layer_type" in HexCoreParams._default_params else HexCoreTransitionLayerType(json_data["transitionLayerType"] if "transitionLayerType" in json_data else None)),
                        cell_element_type if cell_element_type is not None else ( HexCoreParams._default_params["cell_element_type"] if "cell_element_type" in HexCoreParams._default_params else HexCoreCellElementType(json_data["cellElementType"] if "cellElementType" in json_data else None)),
                        surface_mesh_size_scaling if surface_mesh_size_scaling is not None else ( HexCoreParams._default_params["surface_mesh_size_scaling"] if "surface_mesh_size_scaling" in HexCoreParams._default_params else SurfaceMeshSizeScaling(model = model, json_data = (json_data["surfaceMeshSizeScaling"] if "surfaceMeshSizeScaling" in json_data else None))),
                        enable_region_based_hexcore if enable_region_based_hexcore is not None else ( HexCoreParams._default_params["enable_region_based_hexcore"] if "enable_region_based_hexcore" in HexCoreParams._default_params else (json_data["enableRegionBasedHexcore"] if "enableRegionBasedHexcore" in json_data else None)))
        self._custom_params = kwargs
        if model is not None:
            [ model._logger.warning(f'Unsupported argument : {key}') for key in kwargs ]
        [setattr(type(self), key, property(lambda self, key = key:  self._custom_params[key] if key in self._custom_params else None,
        lambda self, value, key = key : self._custom_params.update({ key: value }))) for key in kwargs]
        self._freeze()

    @staticmethod
    def set_default(
            transition_size_field_type: SizeFieldType = None,
            buffer_layers: int = None,
            rel_peel_layer_offset: float = None,
            transition_layer_type: HexCoreTransitionLayerType = None,
            cell_element_type: HexCoreCellElementType = None,
            surface_mesh_size_scaling: SurfaceMeshSizeScaling = None,
            enable_region_based_hexcore: bool = None):
        """Set the default values of HexCoreParams.

        Parameters
        ----------
        transition_size_field_type: SizeFieldType, optional
            Size field type to be used for transition volume (volume between hexcore and boundary).
        buffer_layers: int, optional
            Minimum number of cell layers of the same size before the cell size halves or doubles.
        rel_peel_layer_offset: float, optional
            Gap between hexahedral core and geometry surface relative to the surface mesh size.
        transition_layer_type: HexCoreTransitionLayerType, optional
            Handle size transition of hex cells.
        cell_element_type: HexCoreCellElementType, optional
            Cell element type of hex-shaped cells.
        surface_mesh_size_scaling: SurfaceMeshSizeScaling, optional
            Settings related to scaling of surface mesh size for hexcore refinement.
        enable_region_based_hexcore: bool, optional
            Checks whether to enable region based hexcore or not.
        """
        args = locals()
        [HexCoreParams._default_params.update({ key: value }) for key, value in args.items() if value is not None]

    @staticmethod
    def print_default():
        """Print the default values of HexCoreParams.

        Examples
        --------
        >>> HexCoreParams.print_default()
        """
        message = ""
        message += ''.join(str(key) + ' : ' + str(value) + '\n' for key, value in HexCoreParams._default_params.items())
        print(message)

    def _jsonify(self) -> Dict[str, Any]:
        json_data = {}
        if self._transition_size_field_type is not None:
            json_data["transitionSizeFieldType"] = self._transition_size_field_type
        if self._buffer_layers is not None:
            json_data["bufferLayers"] = self._buffer_layers
        if self._rel_peel_layer_offset is not None:
            json_data["relPeelLayerOffset"] = self._rel_peel_layer_offset
        if self._transition_layer_type is not None:
            json_data["transitionLayerType"] = self._transition_layer_type
        if self._cell_element_type is not None:
            json_data["cellElementType"] = self._cell_element_type
        if self._surface_mesh_size_scaling is not None:
            json_data["surfaceMeshSizeScaling"] = self._surface_mesh_size_scaling._jsonify()
        if self._enable_region_based_hexcore is not None:
            json_data["enableRegionBasedHexcore"] = self._enable_region_based_hexcore
        [ json_data.update({ utils.to_camel_case(key) : value }) for key, value in self._custom_params.items()]
        return json_data

    def __str__(self) -> str:
        message = "transition_size_field_type :  %s\nbuffer_layers :  %s\nrel_peel_layer_offset :  %s\ntransition_layer_type :  %s\ncell_element_type :  %s\nsurface_mesh_size_scaling :  %s\nenable_region_based_hexcore :  %s" % (self._transition_size_field_type, self._buffer_layers, self._rel_peel_layer_offset, self._transition_layer_type, self._cell_element_type, '{ ' + str(self._surface_mesh_size_scaling) + ' }', self._enable_region_based_hexcore)
        message += ''.join('\n' + str(key) + ' : ' + str(value) for key, value in self._custom_params.items())
        return message

    @property
    def transition_size_field_type(self) -> SizeFieldType:
        """Size field type to be used for transition volume (volume between hexcore and boundary).
        """
        return self._transition_size_field_type

    @transition_size_field_type.setter
    def transition_size_field_type(self, value: SizeFieldType):
        self._transition_size_field_type = value

    @property
    def buffer_layers(self) -> int:
        """Minimum number of cell layers of the same size before the cell size halves or doubles.
        """
        return self._buffer_layers

    @buffer_layers.setter
    def buffer_layers(self, value: int):
        self._buffer_layers = value

    @property
    def rel_peel_layer_offset(self) -> float:
        """Gap between hexahedral core and geometry surface relative to the surface mesh size.
        """
        return self._rel_peel_layer_offset

    @rel_peel_layer_offset.setter
    def rel_peel_layer_offset(self, value: float):
        self._rel_peel_layer_offset = value

    @property
    def transition_layer_type(self) -> HexCoreTransitionLayerType:
        """Handle size transition of hex cells.
        """
        return self._transition_layer_type

    @transition_layer_type.setter
    def transition_layer_type(self, value: HexCoreTransitionLayerType):
        self._transition_layer_type = value

    @property
    def cell_element_type(self) -> HexCoreCellElementType:
        """Cell element type of hex-shaped cells.
        """
        return self._cell_element_type

    @cell_element_type.setter
    def cell_element_type(self, value: HexCoreCellElementType):
        self._cell_element_type = value

    @property
    def surface_mesh_size_scaling(self) -> SurfaceMeshSizeScaling:
        """Settings related to scaling of surface mesh size for hexcore refinement.
        """
        return self._surface_mesh_size_scaling

    @surface_mesh_size_scaling.setter
    def surface_mesh_size_scaling(self, value: SurfaceMeshSizeScaling):
        self._surface_mesh_size_scaling = value

    @property
    def enable_region_based_hexcore(self) -> bool:
        """Checks whether to enable region based hexcore or not.
        """
        return self._enable_region_based_hexcore

    @enable_region_based_hexcore.setter
    def enable_region_based_hexcore(self, value: bool):
        self._enable_region_based_hexcore = value

class TetParams(CoreObject):
    """Parameters to control tetrahedral mesh generation.
    """
    _default_params = {}

    def __initialize(
            self,
            quadratic: bool):
        self._quadratic = quadratic

    def __init__(
            self,
            model: CommunicationManager=None,
            quadratic: bool = None,
            json_data : dict = None,
             **kwargs):
        """Initializes the TetParams.

        Parameters
        ----------
        model: Model
            Model to create a TetParams object with default parameters.
        quadratic: bool, optional
            Option to generate quadratic tetrahedral mesh. It is not supported with parallel meshing. It is only supported with pure tetrahedral mesh.
        json_data: dict, optional
            JSON dictionary to create a TetParams object with provided parameters.

        Examples
        --------
        >>> tet_params = prime.TetParams(model = model)
        """
        if json_data:
            self.__initialize(
                json_data["quadratic"] if "quadratic" in json_data else None)
        else:
            all_field_specified = all(arg is not None for arg in [quadratic])
            if all_field_specified:
                self.__initialize(
                    quadratic)
            else:
                if model is None:
                    raise ValueError("Invalid assignment. Either pass model or specify all properties")
                else:
                    param_json = model._communicator.initialize_params(model, "TetParams")
                    json_data = param_json["TetParams"] if "TetParams" in param_json else {}
                    self.__initialize(
                        quadratic if quadratic is not None else ( TetParams._default_params["quadratic"] if "quadratic" in TetParams._default_params else (json_data["quadratic"] if "quadratic" in json_data else None)))
        self._custom_params = kwargs
        if model is not None:
            [ model._logger.warning(f'Unsupported argument : {key}') for key in kwargs ]
        [setattr(type(self), key, property(lambda self, key = key:  self._custom_params[key] if key in self._custom_params else None,
        lambda self, value, key = key : self._custom_params.update({ key: value }))) for key in kwargs]
        self._freeze()

    @staticmethod
    def set_default(
            quadratic: bool = None):
        """Set the default values of TetParams.

        Parameters
        ----------
        quadratic: bool, optional
            Option to generate quadratic tetrahedral mesh. It is not supported with parallel meshing. It is only supported with pure tetrahedral mesh.
        """
        args = locals()
        [TetParams._default_params.update({ key: value }) for key, value in args.items() if value is not None]

    @staticmethod
    def print_default():
        """Print the default values of TetParams.

        Examples
        --------
        >>> TetParams.print_default()
        """
        message = ""
        message += ''.join(str(key) + ' : ' + str(value) + '\n' for key, value in TetParams._default_params.items())
        print(message)

    def _jsonify(self) -> Dict[str, Any]:
        json_data = {}
        if self._quadratic is not None:
            json_data["quadratic"] = self._quadratic
        [ json_data.update({ utils.to_camel_case(key) : value }) for key, value in self._custom_params.items()]
        return json_data

    def __str__(self) -> str:
        message = "quadratic :  %s" % (self._quadratic)
        message += ''.join('\n' + str(key) + ' : ' + str(value) for key, value in self._custom_params.items())
        return message

    @property
    def quadratic(self) -> bool:
        """Option to generate quadratic tetrahedral mesh. It is not supported with parallel meshing. It is only supported with pure tetrahedral mesh.
        """
        return self._quadratic

    @quadratic.setter
    def quadratic(self, value: bool):
        self._quadratic = value

class AutoMeshParams(CoreObject):
    """Parameters for volume meshing.
    """
    _default_params = {}

    def __initialize(
            self,
            size_field_type: SizeFieldType,
            max_size: float,
            prism_control_ids: Iterable[int],
            thin_volume_control_ids: Iterable[int],
            multi_zone_control_ids: Iterable[int],
            volume_fill_type: VolumeFillType,
            prism: PrismParams,
            tet: TetParams,
            hexcore: HexCoreParams,
            volume_control_ids: Iterable[int],
            periodic_control_ids: Iterable[int]):
        self._size_field_type = SizeFieldType(size_field_type)
        self._max_size = max_size
        self._prism_control_ids = prism_control_ids if isinstance(prism_control_ids, np.ndarray) else np.array(prism_control_ids, dtype=np.int32) if prism_control_ids is not None else None
        self._thin_volume_control_ids = thin_volume_control_ids if isinstance(thin_volume_control_ids, np.ndarray) else np.array(thin_volume_control_ids, dtype=np.int32) if thin_volume_control_ids is not None else None
        self._multi_zone_control_ids = multi_zone_control_ids if isinstance(multi_zone_control_ids, np.ndarray) else np.array(multi_zone_control_ids, dtype=np.int32) if multi_zone_control_ids is not None else None
        self._volume_fill_type = VolumeFillType(volume_fill_type)
        self._prism = prism
        self._tet = tet
        self._hexcore = hexcore
        self._volume_control_ids = volume_control_ids if isinstance(volume_control_ids, np.ndarray) else np.array(volume_control_ids, dtype=np.int32) if volume_control_ids is not None else None
        self._periodic_control_ids = periodic_control_ids if isinstance(periodic_control_ids, np.ndarray) else np.array(periodic_control_ids, dtype=np.int32) if periodic_control_ids is not None else None

    def __init__(
            self,
            model: CommunicationManager=None,
            size_field_type: SizeFieldType = None,
            max_size: float = None,
            prism_control_ids: Iterable[int] = None,
            thin_volume_control_ids: Iterable[int] = None,
            multi_zone_control_ids: Iterable[int] = None,
            volume_fill_type: VolumeFillType = None,
            prism: PrismParams = None,
            tet: TetParams = None,
            hexcore: HexCoreParams = None,
            volume_control_ids: Iterable[int] = None,
            periodic_control_ids: Iterable[int] = None,
            json_data : dict = None,
             **kwargs):
        """Initializes the AutoMeshParams.

        Parameters
        ----------
        model: Model
            Model to create a AutoMeshParams object with default parameters.
        size_field_type: SizeFieldType, optional
            Type of sizing to be used to generate volume mesh.
        max_size: float, optional
            Maximum cell size.
        prism_control_ids: Iterable[int], optional
            Set prism control ids.
        thin_volume_control_ids: Iterable[int], optional
            Set thin volume control ids.
        multi_zone_control_ids: Iterable[int], optional
            Set MultiZone control ids.
        volume_fill_type: VolumeFillType, optional
            Option to fill volume.
        prism: PrismParams, optional
            Prism control parameters.
        tet: TetParams, optional
            Parameters to control tetrahedral mesh generation.
        hexcore: HexCoreParams, optional
            Parameters to control hexahedral mesh generation.
        volume_control_ids: Iterable[int], optional
            Ids of the volume controls.
        periodic_control_ids: Iterable[int], optional
            Ids of the periodic controls.
        json_data: dict, optional
            JSON dictionary to create a AutoMeshParams object with provided parameters.

        Examples
        --------
        >>> auto_mesh_params = prime.AutoMeshParams(model = model)
        """
        if json_data:
            self.__initialize(
                SizeFieldType(json_data["sizeFieldType"] if "sizeFieldType" in json_data else None),
                json_data["maxSize"] if "maxSize" in json_data else None,
                json_data["prismControlIds"] if "prismControlIds" in json_data else None,
                json_data["thinVolumeControlIds"] if "thinVolumeControlIds" in json_data else None,
                json_data["multiZoneControlIds"] if "multiZoneControlIds" in json_data else None,
                VolumeFillType(json_data["volumeFillType"] if "volumeFillType" in json_data else None),
                PrismParams(model = model, json_data = json_data["prism"] if "prism" in json_data else None),
                TetParams(model = model, json_data = json_data["tet"] if "tet" in json_data else None),
                HexCoreParams(model = model, json_data = json_data["hexcore"] if "hexcore" in json_data else None),
                json_data["volumeControlIds"] if "volumeControlIds" in json_data else None,
                json_data["periodicControlIds"] if "periodicControlIds" in json_data else None)
        else:
            all_field_specified = all(arg is not None for arg in [size_field_type, max_size, prism_control_ids, thin_volume_control_ids, multi_zone_control_ids, volume_fill_type, prism, tet, hexcore, volume_control_ids, periodic_control_ids])
            if all_field_specified:
                self.__initialize(
                    size_field_type,
                    max_size,
                    prism_control_ids,
                    thin_volume_control_ids,
                    multi_zone_control_ids,
                    volume_fill_type,
                    prism,
                    tet,
                    hexcore,
                    volume_control_ids,
                    periodic_control_ids)
            else:
                if model is None:
                    raise ValueError("Invalid assignment. Either pass model or specify all properties")
                else:
                    param_json = model._communicator.initialize_params(model, "AutoMeshParams")
                    json_data = param_json["AutoMeshParams"] if "AutoMeshParams" in param_json else {}
                    self.__initialize(
                        size_field_type if size_field_type is not None else ( AutoMeshParams._default_params["size_field_type"] if "size_field_type" in AutoMeshParams._default_params else SizeFieldType(json_data["sizeFieldType"] if "sizeFieldType" in json_data else None)),
                        max_size if max_size is not None else ( AutoMeshParams._default_params["max_size"] if "max_size" in AutoMeshParams._default_params else (json_data["maxSize"] if "maxSize" in json_data else None)),
                        prism_control_ids if prism_control_ids is not None else ( AutoMeshParams._default_params["prism_control_ids"] if "prism_control_ids" in AutoMeshParams._default_params else (json_data["prismControlIds"] if "prismControlIds" in json_data else None)),
                        thin_volume_control_ids if thin_volume_control_ids is not None else ( AutoMeshParams._default_params["thin_volume_control_ids"] if "thin_volume_control_ids" in AutoMeshParams._default_params else (json_data["thinVolumeControlIds"] if "thinVolumeControlIds" in json_data else None)),
                        multi_zone_control_ids if multi_zone_control_ids is not None else ( AutoMeshParams._default_params["multi_zone_control_ids"] if "multi_zone_control_ids" in AutoMeshParams._default_params else (json_data["multiZoneControlIds"] if "multiZoneControlIds" in json_data else None)),
                        volume_fill_type if volume_fill_type is not None else ( AutoMeshParams._default_params["volume_fill_type"] if "volume_fill_type" in AutoMeshParams._default_params else VolumeFillType(json_data["volumeFillType"] if "volumeFillType" in json_data else None)),
                        prism if prism is not None else ( AutoMeshParams._default_params["prism"] if "prism" in AutoMeshParams._default_params else PrismParams(model = model, json_data = (json_data["prism"] if "prism" in json_data else None))),
                        tet if tet is not None else ( AutoMeshParams._default_params["tet"] if "tet" in AutoMeshParams._default_params else TetParams(model = model, json_data = (json_data["tet"] if "tet" in json_data else None))),
                        hexcore if hexcore is not None else ( AutoMeshParams._default_params["hexcore"] if "hexcore" in AutoMeshParams._default_params else HexCoreParams(model = model, json_data = (json_data["hexcore"] if "hexcore" in json_data else None))),
                        volume_control_ids if volume_control_ids is not None else ( AutoMeshParams._default_params["volume_control_ids"] if "volume_control_ids" in AutoMeshParams._default_params else (json_data["volumeControlIds"] if "volumeControlIds" in json_data else None)),
                        periodic_control_ids if periodic_control_ids is not None else ( AutoMeshParams._default_params["periodic_control_ids"] if "periodic_control_ids" in AutoMeshParams._default_params else (json_data["periodicControlIds"] if "periodicControlIds" in json_data else None)))
        self._custom_params = kwargs
        if model is not None:
            [ model._logger.warning(f'Unsupported argument : {key}') for key in kwargs ]
        [setattr(type(self), key, property(lambda self, key = key:  self._custom_params[key] if key in self._custom_params else None,
        lambda self, value, key = key : self._custom_params.update({ key: value }))) for key in kwargs]
        self._freeze()

    @staticmethod
    def set_default(
            size_field_type: SizeFieldType = None,
            max_size: float = None,
            prism_control_ids: Iterable[int] = None,
            thin_volume_control_ids: Iterable[int] = None,
            multi_zone_control_ids: Iterable[int] = None,
            volume_fill_type: VolumeFillType = None,
            prism: PrismParams = None,
            tet: TetParams = None,
            hexcore: HexCoreParams = None,
            volume_control_ids: Iterable[int] = None,
            periodic_control_ids: Iterable[int] = None):
        """Set the default values of AutoMeshParams.

        Parameters
        ----------
        size_field_type: SizeFieldType, optional
            Type of sizing to be used to generate volume mesh.
        max_size: float, optional
            Maximum cell size.
        prism_control_ids: Iterable[int], optional
            Set prism control ids.
        thin_volume_control_ids: Iterable[int], optional
            Set thin volume control ids.
        multi_zone_control_ids: Iterable[int], optional
            Set MultiZone control ids.
        volume_fill_type: VolumeFillType, optional
            Option to fill volume.
        prism: PrismParams, optional
            Prism control parameters.
        tet: TetParams, optional
            Parameters to control tetrahedral mesh generation.
        hexcore: HexCoreParams, optional
            Parameters to control hexahedral mesh generation.
        volume_control_ids: Iterable[int], optional
            Ids of the volume controls.
        periodic_control_ids: Iterable[int], optional
            Ids of the periodic controls.
        """
        args = locals()
        [AutoMeshParams._default_params.update({ key: value }) for key, value in args.items() if value is not None]

    @staticmethod
    def print_default():
        """Print the default values of AutoMeshParams.

        Examples
        --------
        >>> AutoMeshParams.print_default()
        """
        message = ""
        message += ''.join(str(key) + ' : ' + str(value) + '\n' for key, value in AutoMeshParams._default_params.items())
        print(message)

    def _jsonify(self) -> Dict[str, Any]:
        json_data = {}
        if self._size_field_type is not None:
            json_data["sizeFieldType"] = self._size_field_type
        if self._max_size is not None:
            json_data["maxSize"] = self._max_size
        if self._prism_control_ids is not None:
            json_data["prismControlIds"] = self._prism_control_ids
        if self._thin_volume_control_ids is not None:
            json_data["thinVolumeControlIds"] = self._thin_volume_control_ids
        if self._multi_zone_control_ids is not None:
            json_data["multiZoneControlIds"] = self._multi_zone_control_ids
        if self._volume_fill_type is not None:
            json_data["volumeFillType"] = self._volume_fill_type
        if self._prism is not None:
            json_data["prism"] = self._prism._jsonify()
        if self._tet is not None:
            json_data["tet"] = self._tet._jsonify()
        if self._hexcore is not None:
            json_data["hexcore"] = self._hexcore._jsonify()
        if self._volume_control_ids is not None:
            json_data["volumeControlIds"] = self._volume_control_ids
        if self._periodic_control_ids is not None:
            json_data["periodicControlIds"] = self._periodic_control_ids
        [ json_data.update({ utils.to_camel_case(key) : value }) for key, value in self._custom_params.items()]
        return json_data

    def __str__(self) -> str:
        message = "size_field_type :  %s\nmax_size :  %s\nprism_control_ids :  %s\nthin_volume_control_ids :  %s\nmulti_zone_control_ids :  %s\nvolume_fill_type :  %s\nprism :  %s\ntet :  %s\nhexcore :  %s\nvolume_control_ids :  %s\nperiodic_control_ids :  %s" % (self._size_field_type, self._max_size, self._prism_control_ids, self._thin_volume_control_ids, self._multi_zone_control_ids, self._volume_fill_type, '{ ' + str(self._prism) + ' }', '{ ' + str(self._tet) + ' }', '{ ' + str(self._hexcore) + ' }', self._volume_control_ids, self._periodic_control_ids)
        message += ''.join('\n' + str(key) + ' : ' + str(value) for key, value in self._custom_params.items())
        return message

    @property
    def size_field_type(self) -> SizeFieldType:
        """Type of sizing to be used to generate volume mesh.
        """
        return self._size_field_type

    @size_field_type.setter
    def size_field_type(self, value: SizeFieldType):
        self._size_field_type = value

    @property
    def max_size(self) -> float:
        """Maximum cell size.
        """
        return self._max_size

    @max_size.setter
    def max_size(self, value: float):
        self._max_size = value

    @property
    def prism_control_ids(self) -> Iterable[int]:
        """Set prism control ids.
        """
        return self._prism_control_ids

    @prism_control_ids.setter
    def prism_control_ids(self, value: Iterable[int]):
        self._prism_control_ids = value

    @property
    def thin_volume_control_ids(self) -> Iterable[int]:
        """Set thin volume control ids.
        """
        return self._thin_volume_control_ids

    @thin_volume_control_ids.setter
    def thin_volume_control_ids(self, value: Iterable[int]):
        self._thin_volume_control_ids = value

    @property
    def multi_zone_control_ids(self) -> Iterable[int]:
        """Set MultiZone control ids.
        """
        return self._multi_zone_control_ids

    @multi_zone_control_ids.setter
    def multi_zone_control_ids(self, value: Iterable[int]):
        self._multi_zone_control_ids = value

    @property
    def volume_fill_type(self) -> VolumeFillType:
        """Option to fill volume.
        """
        return self._volume_fill_type

    @volume_fill_type.setter
    def volume_fill_type(self, value: VolumeFillType):
        self._volume_fill_type = value

    @property
    def prism(self) -> PrismParams:
        """Prism control parameters.
        """
        return self._prism

    @prism.setter
    def prism(self, value: PrismParams):
        self._prism = value

    @property
    def tet(self) -> TetParams:
        """Parameters to control tetrahedral mesh generation.
        """
        return self._tet

    @tet.setter
    def tet(self, value: TetParams):
        self._tet = value

    @property
    def hexcore(self) -> HexCoreParams:
        """Parameters to control hexahedral mesh generation.
        """
        return self._hexcore

    @hexcore.setter
    def hexcore(self, value: HexCoreParams):
        self._hexcore = value

    @property
    def volume_control_ids(self) -> Iterable[int]:
        """Ids of the volume controls.
        """
        return self._volume_control_ids

    @volume_control_ids.setter
    def volume_control_ids(self, value: Iterable[int]):
        self._volume_control_ids = value

    @property
    def periodic_control_ids(self) -> Iterable[int]:
        """Ids of the periodic controls.
        """
        return self._periodic_control_ids

    @periodic_control_ids.setter
    def periodic_control_ids(self, value: Iterable[int]):
        self._periodic_control_ids = value



"""Module containing classes and methods related to periodic control."""

from ansys.meshing.prime.autogen.periodiccontrol import (
    PeriodicControl as _PeriodicControl,
)

# isort: split
from ansys.meshing.prime.autogen.commonstructs import SetNameResults
from ansys.meshing.prime.autogen.periodiccontrolstructs import PeriodicControlParams


class PeriodicControl(_PeriodicControl):
    """Defines the scope and transformation for periodic surfaces.

    Parameters
    ----------
    model : CommunicationManager
        Server model to create and modify periodic controls from.
    id : int
        ID of the control.
    object_id : int
        Object ID of the control.
    name : str
        Name of the periodic control.
    local : bool, optional
        Unused. The default is ``False``.
    """

    def __init__(self, model, id, object_id, name, local=False):
        """Initialize class variables and the superclass."""
        _PeriodicControl.__init__(self, model, id, object_id, name)
        self._model = model
        self._name = name

    def __str__(self) -> str:
        """Get a representation of the class in string format.

        Returns
        -------
        str
            Class data in string format.
        """
        params = PeriodicControlParams(model=self._model)
        result = _PeriodicControl.get_summary(self, params)
        return result.message

    def set_suggested_name(self, name: str) -> SetNameResults:
        """Set the unique name for the periodic control to a suggested name.

        Parameters
        ----------
        name : str
            Suggested name for the periodic control.

        Returns
        -------
        SetNameResults
            Newly suggested name of the periodic control.

        Examples
        --------
        >>> periodic_control.set_suggested_name("control1")

        """
        result = _PeriodicControl.set_suggested_name(self, name)
        self._name = result.assigned_name
        return result

    @property
    def name(self):
        """Get the name of the periodic control.

        Returns
        -------
        str
            Name of the periodic control.

        Examples
        --------
        >>> print(periodic_control.name)

        """
        return self._name



"""Module containing the managing logic of the Prime model."""
from typing import Iterable, List

# isort: split
from ansys.meshing.prime.autogen.model import Model as _Model

# isort: split
import ansys.meshing.prime.internals.json_utils as json
from ansys.meshing.prime.autogen.commonstructs import DeleteResults
from ansys.meshing.prime.autogen.materialpointmanager import MaterialPointManager
from ansys.meshing.prime.autogen.modelstructs import (
    GlobalSizingParams,
    MergePartsParams,
    MergePartsResults,
)
from ansys.meshing.prime.autogen.primeconfig import ErrorCode
from ansys.meshing.prime.autogen.topodata import TopoData
from ansys.meshing.prime.core.controldata import ControlData
from ansys.meshing.prime.core.part import Part
from ansys.meshing.prime.internals.communicator import Communicator
from ansys.meshing.prime.internals.error_handling import PrimeRuntimeError
from ansys.meshing.prime.internals.logger import PrimeLogger


class Model(_Model):
    """Contains all information about Ansys Prime Server.

    This class provides the nucleus of Ansys meshing technology. You can access
    any information in Ansys Prime Server only through the ``Model`` class.
    Using this class, you can query topo data, control data, parts, size fields,
    and more.

    Parameters
    ----------
    comm : Communicator
        Communicator to connect with the Ansys Prime server.
    id : int
        ID of the model.
    object_id : int
        Object ID of the model.
    name : str
        Name of the model.
    """

    __doc__ = _Model.__doc__

    def __init__(self, comm: Communicator, id: int, object_id: int, name: str):
        """Initialize the model and the parameters."""
        _Model.__init__(self, comm, id, object_id, name)
        self._parts = []
        self._global_sf_params = GlobalSizingParams(model=self)
        self._default_part = None
        self._topo_data = None
        self._control_data = None
        self._material_point_data = None
        self._freeze()

    def _sync_up_model(self):
        """Synchronize the client model with the server model.

        This method Updates proxy child objects of the client model with the
        child objects of the server model.

        Examples
        --------
        >>> from ansys.meshing.prime import Model
        >>> model = client.model
        >>> model._sync_up_model()
        """
        res = json.loads(
            self._comm.serve(self, "PrimeMesh::Model/GetChildObjectsJson", self._object_id, args={})
        )
        part_data = res["Parts"]
        sc_data = res["SizeControl"]
        pc_data = res["PrismControl"]
        wc_data = res["WrapperControl"]
        mc_data = res["WrapperControl"]
        vc_data = res["VolumeControl"]
        if "PeriodicControl" in res:
            percon_data = res["PeriodicControl"]
        sf_params = res["GlobalSizingParams"]

        self._global_sf_params = GlobalSizingParams(
            model=self, min=sf_params[0], max=sf_params[1], growth_rate=sf_params[2]
        )
        self._parts = [Part(self, part[0], part[1], part[2]) for part in part_data]
        self._control_data = ControlData(self, -1, res["ControlData"], "")
        self._control_data._update_size_controls(sc_data)
        self._control_data._update_prism_controls(pc_data)
        self._control_data._update_wrapper_controls(wc_data)
        self._control_data._update_multi_zone_controls(mc_data)
        self._control_data._update_volume_controls(vc_data)
        self._topo_data = TopoData(self, -1, res["TopoData"], "")
        if "PeriodicControl" in res:
            self._control_data._update_periodic_controls(percon_data)
        self._material_point_data = MaterialPointManager(self, -1, res["MaterialPointData"], "")

    def _add_part(self, id: int):
        """Add a part that is present on the server.

        Parameters
        ----------
        id : int
            ID of the part.

        Raises
        ------
        PrimeRuntimeError
            Raise if unable to create the part.
        """
        res = json.loads(
            self._comm.serve(self, "PrimeMesh::Model/GetChildObjectsJson", self._object_id, args={})
        )
        part_data = res["Parts"]
        new_part = None
        for part in part_data:
            if part[0] == id:
                new_part = Part(self, part[0], part[1], part[2])
                self._parts.append(new_part)
                break
        if new_part == None:
            raise PrimeRuntimeError("Unable to create part", ErrorCode.PARTNOTFOUND)

    def get_part_by_name(self, name: str) -> Part:
        """Get the part by name.

        Parameters
        ----------
        name : str
            Name of the part.

        Returns
        -------
        Part
            Part or ``None`` if the given part name doesn't exist.

        Examples
        --------
            >>> from ansys.meshing.prime import Model
            >>> model = client.model
            >>> part = model.get_part_by_name("part.1")
        """
        for part in self._parts:
            if part.name == name:
                return part
        return None

    def get_part(self, id: int) -> Part:
        """Get the part by ID.

        Parameters
        ----------
        id : int
            ID of the part.

        Returns
        -------
        Part
            Part or ``None`` if the given part ID doesn't exist.

        Examples
        --------
            >>> from ansys.meshing.prime import Model
            >>> model = client.model
            >>> part = model.get_part(2)
        """
        for part in self._parts:
            if part.id == id:
                return part
        return None

    def merge_parts(self, part_ids: Iterable[int], params: MergePartsParams) -> MergePartsResults:
        """Merge multiple parts into a single part.

        Parameters
        ----------
        part_ids : Iterable[int]
            IDs of the parts to merge.
        params : MergePartsParams
            Parameters for merging parts.

        Returns
        -------
        MergePartsResults
            Results for merging the parts into a single part.


        Examples
        --------
        >>> params = prime.MergePartsParams(model = model)
        >>> results = model.merge_parts(part_ids, params)

        """
        res = _Model.merge_parts(self, part_ids, params)
        merged_part = self.get_part_by_name(res.merged_part_assigned_name)
        if merged_part is None:
            self._sync_up_model()
        else:
            for id in part_ids:
                part = self.get_part(id)
                if part.id != merged_part.id:
                    self._parts.remove(part)
        return res

    def delete_parts(self, part_ids: Iterable[int]) -> DeleteResults:
        """Delete parts and their contents.

        Parameters
        ----------
        part_ids : Iterable[int]
            IDs of parts to delete.

        Returns
        -------
        DeleteResults
            Results from deleting parts and their contents.


        Examples
        --------
        >>> results = model.delete_parts(part_ids)

        """
        res = _Model.delete_parts(self, part_ids)
        if res.error_code == ErrorCode.NOERROR:
            for id in part_ids:
                for part in list(self._parts):
                    if part.id == id:
                        self._parts.remove(part)
        return res

    def get_global_sizing_params(self) -> GlobalSizingParams:
        """Get global sizing parameters.

        Returns
        -------
        GlobalSizingParams
            Global sizing parameters.

        Examples
        --------
            >>> model = client.model
            >>> sf_params = model.get_global_sizing_params()
        """
        return self._global_sf_params

    def set_global_sizing_params(self, params: GlobalSizingParams):
        """Set global sizing parameters.

        Parameters
        ----------
        params : GlobalSizingParams
            Global sizing parameters for initializing surfer parameters and
            various size control parameters.

        Examples
        --------
        >>> model = client.model
        >>> model.set_global_sizing_params(GlobalSizingParams(model=model,
        ...                                          min=0.1,
        ...                                          max=1.0,
        ...                                          growth_rate=1.2))
        """
        _Model.set_global_sizing_params(self, params)
        self._global_sf_params = params

    def __str__(self):
        """Print the summary of the model.

        Returns
        -------
        str
            Summary of the model.

        Examples
        --------
        >>> from ansys.meshing.prime import Model
        >>> model = client.model
        >>> print(model)
        """
        result = ""
        result += "Part Summary:\n"
        for part in self._parts:
            result += part.__str__() + "\n"
        return result

    @property
    def parts(self) -> List[Part]:
        """Get the list of parts for the model.

        Returns
        -------
        List[Part]
            List of parts for the model.

        Examples
        --------
            >>> from ansys.meshing.prime import Model
            >>> model = client.model
            >>> parts = model.parts
        """
        return self._parts

    @property
    def topo_data(self) -> TopoData:
        """Get the TopoData for the model.

        Returns
        -------
        TopoData
            TopoData for the model.

        Examples
        --------
            >>> topo_data=model.topo_data
        """
        return self._topo_data

    @property
    def control_data(self) -> ControlData:
        """Get the control data for the model.

        Returns
        -------
        ControlData
            Control data for the model.

        Examples
        --------
            >>> control_data = model.control_data
        """
        if self._control_data is None:
            self._sync_up_model()
        return self._control_data

    @property
    def material_point_data(self) -> MaterialPointManager:
        """Get material point data for the model.

        The Material Point Manager is used to create, delete, and manage material points.

        Returns
        -------
        MaterialPointManager
            Material Point Manager.

        Examples
        --------
            >>> mpt_data = model.material_point_data
        """
        if self._material_point_data is None:
            self._sync_up_model()
        return self._material_point_data

    @property
    def python_logger(self):
        """Get PyPrimeMesh's logger instance.

        PyPrimeMesh's logger instance can be used to control the verbosity
        of messages printed by PyPrimeMesh.

        Returns
        -------
        logging.Logger
            PyPrimeMesh's logger instance.

        Examples
        --------
        Set log level to debug.

        >>> model.python_logger.setLevel(logging.DEBUG)

        """
        return PrimeLogger().get_logger()



""" Auto-generated file. DO NOT MODIFY """
import enum
from typing import Dict, Any, Union, List, Iterable
from ansys.meshing.prime.internals.comm_manager import CommunicationManager
from ansys.meshing.prime.internals import utils
from ansys.meshing.prime.autogen.coreobject import *
import numpy as np

from ansys.meshing.prime.params.primestructs import *

class BCPairType(enum.IntEnum):
    """Option to specify boundary condition pair type.
    """
    FACE = 1
    """Option to specify face zonelet as boundary condition pair."""
    EDGE = 2
    """Option to specify edge zonelet as boundary condition pair."""

class MatchPairTargetType(enum.IntEnum):
    """Match morph target type.
    """
    FACEZONELET = 1
    """Option to specify face zonelet as target."""
    TOPOFACE = 2
    """Option to specify topoface as target."""

class MorphSolveParams(CoreObject):
    """Morpher solve parameters.
    """
    _default_params = {}

    def __initialize(
            self):
        pass

    def __init__(
            self,
            model: CommunicationManager=None,
            json_data : dict = None,
             **kwargs):
        """Initializes the MorphSolveParams.

        Parameters
        ----------
        model: Model
            Model to create a MorphSolveParams object with default parameters.
        json_data: dict, optional
            JSON dictionary to create a MorphSolveParams object with provided parameters.

        Examples
        --------
        >>> morph_solve_params = prime.MorphSolveParams(model = model)
        """
        if json_data:
            self.__initialize()
        else:
            all_field_specified = all(arg is not None for arg in [])
            if all_field_specified:
                self.__initialize()
            else:
                if model is None:
                    raise ValueError("Invalid assignment. Either pass model or specify all properties")
                else:
                    param_json = model._communicator.initialize_params(model, "MorphSolveParams")
                    json_data = param_json["MorphSolveParams"] if "MorphSolveParams" in param_json else {}
                    self.__initialize()
        self._custom_params = kwargs
        if model is not None:
            [ model._logger.warning(f'Unsupported argument : {key}') for key in kwargs ]
        [setattr(type(self), key, property(lambda self, key = key:  self._custom_params[key] if key in self._custom_params else None,
        lambda self, value, key = key : self._custom_params.update({ key: value }))) for key in kwargs]
        self._freeze()

    @staticmethod
    def set_default():
        """Set the default values of MorphSolveParams.

        """
        args = locals()
        [MorphSolveParams._default_params.update({ key: value }) for key, value in args.items() if value is not None]

    @staticmethod
    def print_default():
        """Print the default values of MorphSolveParams.

        Examples
        --------
        >>> MorphSolveParams.print_default()
        """
        message = ""
        message += ''.join(str(key) + ' : ' + str(value) + '\n' for key, value in MorphSolveParams._default_params.items())
        print(message)

    def _jsonify(self) -> Dict[str, Any]:
        json_data = {}
        [ json_data.update({ utils.to_camel_case(key) : value }) for key, value in self._custom_params.items()]
        return json_data

    def __str__(self) -> str:
        message = "" % ()
        message += ''.join('\n' + str(key) + ' : ' + str(value) for key, value in self._custom_params.items())
        return message

class MatchMorphParams(CoreObject):
    """MatchMorphParams describes the parameters required for match morphing.
    """
    _default_params = {}

    def __initialize(
            self):
        pass

    def __init__(
            self,
            model: CommunicationManager=None,
            json_data : dict = None,
             **kwargs):
        """Initializes the MatchMorphParams.

        Parameters
        ----------
        model: Model
            Model to create a MatchMorphParams object with default parameters.
        json_data: dict, optional
            JSON dictionary to create a MatchMorphParams object with provided parameters.

        Examples
        --------
        >>> match_morph_params = prime.MatchMorphParams(model = model)
        """
        if json_data:
            self.__initialize()
        else:
            all_field_specified = all(arg is not None for arg in [])
            if all_field_specified:
                self.__initialize()
            else:
                if model is None:
                    raise ValueError("Invalid assignment. Either pass model or specify all properties")
                else:
                    param_json = model._communicator.initialize_params(model, "MatchMorphParams")
                    json_data = param_json["MatchMorphParams"] if "MatchMorphParams" in param_json else {}
                    self.__initialize()
        self._custom_params = kwargs
        if model is not None:
            [ model._logger.warning(f'Unsupported argument : {key}') for key in kwargs ]
        [setattr(type(self), key, property(lambda self, key = key:  self._custom_params[key] if key in self._custom_params else None,
        lambda self, value, key = key : self._custom_params.update({ key: value }))) for key in kwargs]
        self._freeze()

    @staticmethod
    def set_default():
        """Set the default values of MatchMorphParams.

        """
        args = locals()
        [MatchMorphParams._default_params.update({ key: value }) for key, value in args.items() if value is not None]

    @staticmethod
    def print_default():
        """Print the default values of MatchMorphParams.

        Examples
        --------
        >>> MatchMorphParams.print_default()
        """
        message = ""
        message += ''.join(str(key) + ' : ' + str(value) + '\n' for key, value in MatchMorphParams._default_params.items())
        print(message)

    def _jsonify(self) -> Dict[str, Any]:
        json_data = {}
        [ json_data.update({ utils.to_camel_case(key) : value }) for key, value in self._custom_params.items()]
        return json_data

    def __str__(self) -> str:
        message = "" % ()
        message += ''.join('\n' + str(key) + ' : ' + str(value) for key, value in self._custom_params.items())
        return message

class MatchMorphResults(CoreObject):
    """Results associated with match morph operation.
    """
    _default_params = {}

    def __initialize(
            self,
            error_code: ErrorCode):
        self._error_code = ErrorCode(error_code)

    def __init__(
            self,
            model: CommunicationManager=None,
            error_code: ErrorCode = None,
            json_data : dict = None,
             **kwargs):
        """Initializes the MatchMorphResults.

        Parameters
        ----------
        model: Model
            Model to create a MatchMorphResults object with default parameters.
        error_code: ErrorCode, optional
            Errorcode associated with match morph operation.
        json_data: dict, optional
            JSON dictionary to create a MatchMorphResults object with provided parameters.

        Examples
        --------
        >>> match_morph_results = prime.MatchMorphResults(model = model)
        """
        if json_data:
            self.__initialize(
                ErrorCode(json_data["errorCode"] if "errorCode" in json_data else None))
        else:
            all_field_specified = all(arg is not None for arg in [error_code])
            if all_field_specified:
                self.__initialize(
                    error_code)
            else:
                if model is None:
                    raise ValueError("Invalid assignment. Either pass model or specify all properties")
                else:
                    param_json = model._communicator.initialize_params(model, "MatchMorphResults")
                    json_data = param_json["MatchMorphResults"] if "MatchMorphResults" in param_json else {}
                    self.__initialize(
                        error_code if error_code is not None else ( MatchMorphResults._default_params["error_code"] if "error_code" in MatchMorphResults._default_params else ErrorCode(json_data["errorCode"] if "errorCode" in json_data else None)))
        self._custom_params = kwargs
        if model is not None:
            [ model._logger.warning(f'Unsupported argument : {key}') for key in kwargs ]
        [setattr(type(self), key, property(lambda self, key = key:  self._custom_params[key] if key in self._custom_params else None,
        lambda self, value, key = key : self._custom_params.update({ key: value }))) for key in kwargs]
        self._freeze()

    @staticmethod
    def set_default(
            error_code: ErrorCode = None):
        """Set the default values of MatchMorphResults.

        Parameters
        ----------
        error_code: ErrorCode, optional
            Errorcode associated with match morph operation.
        """
        args = locals()
        [MatchMorphResults._default_params.update({ key: value }) for key, value in args.items() if value is not None]

    @staticmethod
    def print_default():
        """Print the default values of MatchMorphResults.

        Examples
        --------
        >>> MatchMorphResults.print_default()
        """
        message = ""
        message += ''.join(str(key) + ' : ' + str(value) + '\n' for key, value in MatchMorphResults._default_params.items())
        print(message)

    def _jsonify(self) -> Dict[str, Any]:
        json_data = {}
        if self._error_code is not None:
            json_data["errorCode"] = self._error_code
        [ json_data.update({ utils.to_camel_case(key) : value }) for key, value in self._custom_params.items()]
        return json_data

    def __str__(self) -> str:
        message = "error_code :  %s" % (self._error_code)
        message += ''.join('\n' + str(key) + ' : ' + str(value) for key, value in self._custom_params.items())
        return message

    @property
    def error_code(self) -> ErrorCode:
        """Errorcode associated with match morph operation.
        """
        return self._error_code

    @error_code.setter
    def error_code(self, value: ErrorCode):
        self._error_code = value

class BCPair(CoreObject):
    """Used to define boundary conditions for match morphing.
    """
    _default_params = {}

    def __initialize(
            self,
            source_zonelet: int,
            target_zonelet: int,
            type: BCPairType):
        self._source_zonelet = source_zonelet
        self._target_zonelet = target_zonelet
        self._type = BCPairType(type)

    def __init__(
            self,
            model: CommunicationManager=None,
            source_zonelet: int = None,
            target_zonelet: int = None,
            type: BCPairType = None,
            json_data : dict = None,
             **kwargs):
        """Initializes the BCPair.

        Parameters
        ----------
        model: Model
            Model to create a BCPair object with default parameters.
        source_zonelet: int, optional
            Id of source zonelet.
        target_zonelet: int, optional
            Id of target zonelet.
        type: BCPairType, optional
            Option to specify boundary condition pair type.
        json_data: dict, optional
            JSON dictionary to create a BCPair object with provided parameters.

        Examples
        --------
        >>> b_cpair = prime.BCPair(model = model)
        """
        if json_data:
            self.__initialize(
                json_data["sourceZonelet"] if "sourceZonelet" in json_data else None,
                json_data["targetZonelet"] if "targetZonelet" in json_data else None,
                BCPairType(json_data["type"] if "type" in json_data else None))
        else:
            all_field_specified = all(arg is not None for arg in [source_zonelet, target_zonelet, type])
            if all_field_specified:
                self.__initialize(
                    source_zonelet,
                    target_zonelet,
                    type)
            else:
                if model is None:
                    raise ValueError("Invalid assignment. Either pass model or specify all properties")
                else:
                    param_json = model._communicator.initialize_params(model, "BCPair")
                    json_data = param_json["BCPair"] if "BCPair" in param_json else {}
                    self.__initialize(
                        source_zonelet if source_zonelet is not None else ( BCPair._default_params["source_zonelet"] if "source_zonelet" in BCPair._default_params else (json_data["sourceZonelet"] if "sourceZonelet" in json_data else None)),
                        target_zonelet if target_zonelet is not None else ( BCPair._default_params["target_zonelet"] if "target_zonelet" in BCPair._default_params else (json_data["targetZonelet"] if "targetZonelet" in json_data else None)),
                        type if type is not None else ( BCPair._default_params["type"] if "type" in BCPair._default_params else BCPairType(json_data["type"] if "type" in json_data else None)))
        self._custom_params = kwargs
        if model is not None:
            [ model._logger.warning(f'Unsupported argument : {key}') for key in kwargs ]
        [setattr(type(self), key, property(lambda self, key = key:  self._custom_params[key] if key in self._custom_params else None,
        lambda self, value, key = key : self._custom_params.update({ key: value }))) for key in kwargs]
        self._freeze()

    @staticmethod
    def set_default(
            source_zonelet: int = None,
            target_zonelet: int = None,
            type: BCPairType = None):
        """Set the default values of BCPair.

        Parameters
        ----------
        source_zonelet: int, optional
            Id of source zonelet.
        target_zonelet: int, optional
            Id of target zonelet.
        type: BCPairType, optional
            Option to specify boundary condition pair type.
        """
        args = locals()
        [BCPair._default_params.update({ key: value }) for key, value in args.items() if value is not None]

    @staticmethod
    def print_default():
        """Print the default values of BCPair.

        Examples
        --------
        >>> BCPair.print_default()
        """
        message = ""
        message += ''.join(str(key) + ' : ' + str(value) + '\n' for key, value in BCPair._default_params.items())
        print(message)

    def _jsonify(self) -> Dict[str, Any]:
        json_data = {}
        if self._source_zonelet is not None:
            json_data["sourceZonelet"] = self._source_zonelet
        if self._target_zonelet is not None:
            json_data["targetZonelet"] = self._target_zonelet
        if self._type is not None:
            json_data["type"] = self._type
        [ json_data.update({ utils.to_camel_case(key) : value }) for key, value in self._custom_params.items()]
        return json_data

    def __str__(self) -> str:
        message = "source_zonelet :  %s\ntarget_zonelet :  %s\ntype :  %s" % (self._source_zonelet, self._target_zonelet, self._type)
        message += ''.join('\n' + str(key) + ' : ' + str(value) for key, value in self._custom_params.items())
        return message

    @property
    def source_zonelet(self) -> int:
        """Id of source zonelet.
        """
        return self._source_zonelet

    @source_zonelet.setter
    def source_zonelet(self, value: int):
        self._source_zonelet = value

    @property
    def target_zonelet(self) -> int:
        """Id of target zonelet.
        """
        return self._target_zonelet

    @target_zonelet.setter
    def target_zonelet(self, value: int):
        self._target_zonelet = value

    @property
    def type(self) -> BCPairType:
        """Option to specify boundary condition pair type.
        """
        return self._type

    @type.setter
    def type(self, value: BCPairType):
        self._type = value

class MatchPair(CoreObject):
    """Match pair to specify sources, targets for match morphing. Included boundary conditions specification.
    """
    _default_params = {}

    def __initialize(
            self,
            source_surfaces: Iterable[int],
            target_surfaces: Iterable[int],
            target_type: MatchPairTargetType,
            bc_pairs: List[BCPair]):
        self._source_surfaces = source_surfaces if isinstance(source_surfaces, np.ndarray) else np.array(source_surfaces, dtype=np.int32) if source_surfaces is not None else None
        self._target_surfaces = target_surfaces if isinstance(target_surfaces, np.ndarray) else np.array(target_surfaces, dtype=np.int32) if target_surfaces is not None else None
        self._target_type = MatchPairTargetType(target_type)
        self._bc_pairs = bc_pairs

    def __init__(
            self,
            model: CommunicationManager=None,
            source_surfaces: Iterable[int] = None,
            target_surfaces: Iterable[int] = None,
            target_type: MatchPairTargetType = None,
            bc_pairs: List[BCPair] = None,
            json_data : dict = None,
             **kwargs):
        """Initializes the MatchPair.

        Parameters
        ----------
        model: Model
            Model to create a MatchPair object with default parameters.
        source_surfaces: Iterable[int], optional
            Ids of source surfaces.
        target_surfaces: Iterable[int], optional
            Ids of target surfaces.
        target_type: MatchPairTargetType, optional
            Option to specify target surface type.
        bc_pairs: List[BCPair], optional
            Array of boundary condition pairs.
        json_data: dict, optional
            JSON dictionary to create a MatchPair object with provided parameters.

        Examples
        --------
        >>> match_pair = prime.MatchPair(model = model)
        """
        if json_data:
            self.__initialize(
                json_data["sourceSurfaces"] if "sourceSurfaces" in json_data else None,
                json_data["targetSurfaces"] if "targetSurfaces" in json_data else None,
                MatchPairTargetType(json_data["targetType"] if "targetType" in json_data else None),
                [BCPair(model = model, json_data = data) for data in json_data["bcPairs"]] if "bcPairs" in json_data else None)
        else:
            all_field_specified = all(arg is not None for arg in [source_surfaces, target_surfaces, target_type, bc_pairs])
            if all_field_specified:
                self.__initialize(
                    source_surfaces,
                    target_surfaces,
                    target_type,
                    bc_pairs)
            else:
                if model is None:
                    raise ValueError("Invalid assignment. Either pass model or specify all properties")
                else:
                    param_json = model._communicator.initialize_params(model, "MatchPair")
                    json_data = param_json["MatchPair"] if "MatchPair" in param_json else {}
                    self.__initialize(
                        source_surfaces if source_surfaces is not None else ( MatchPair._default_params["source_surfaces"] if "source_surfaces" in MatchPair._default_params else (json_data["sourceSurfaces"] if "sourceSurfaces" in json_data else None)),
                        target_surfaces if target_surfaces is not None else ( MatchPair._default_params["target_surfaces"] if "target_surfaces" in MatchPair._default_params else (json_data["targetSurfaces"] if "targetSurfaces" in json_data else None)),
                        target_type if target_type is not None else ( MatchPair._default_params["target_type"] if "target_type" in MatchPair._default_params else MatchPairTargetType(json_data["targetType"] if "targetType" in json_data else None)),
                        bc_pairs if bc_pairs is not None else ( MatchPair._default_params["bc_pairs"] if "bc_pairs" in MatchPair._default_params else [BCPair(model = model, json_data = data) for data in (json_data["bcPairs"] if "bcPairs" in json_data else None)]))
        self._custom_params = kwargs
        if model is not None:
            [ model._logger.warning(f'Unsupported argument : {key}') for key in kwargs ]
        [setattr(type(self), key, property(lambda self, key = key:  self._custom_params[key] if key in self._custom_params else None,
        lambda self, value, key = key : self._custom_params.update({ key: value }))) for key in kwargs]
        self._freeze()

    @staticmethod
    def set_default(
            source_surfaces: Iterable[int] = None,
            target_surfaces: Iterable[int] = None,
            target_type: MatchPairTargetType = None,
            bc_pairs: List[BCPair] = None):
        """Set the default values of MatchPair.

        Parameters
        ----------
        source_surfaces: Iterable[int], optional
            Ids of source surfaces.
        target_surfaces: Iterable[int], optional
            Ids of target surfaces.
        target_type: MatchPairTargetType, optional
            Option to specify target surface type.
        bc_pairs: List[BCPair], optional
            Array of boundary condition pairs.
        """
        args = locals()
        [MatchPair._default_params.update({ key: value }) for key, value in args.items() if value is not None]

    @staticmethod
    def print_default():
        """Print the default values of MatchPair.

        Examples
        --------
        >>> MatchPair.print_default()
        """
        message = ""
        message += ''.join(str(key) + ' : ' + str(value) + '\n' for key, value in MatchPair._default_params.items())
        print(message)

    def _jsonify(self) -> Dict[str, Any]:
        json_data = {}
        if self._source_surfaces is not None:
            json_data["sourceSurfaces"] = self._source_surfaces
        if self._target_surfaces is not None:
            json_data["targetSurfaces"] = self._target_surfaces
        if self._target_type is not None:
            json_data["targetType"] = self._target_type
        if self._bc_pairs is not None:
            json_data["bcPairs"] = [data._jsonify() for data in self._bc_pairs]
        [ json_data.update({ utils.to_camel_case(key) : value }) for key, value in self._custom_params.items()]
        return json_data

    def __str__(self) -> str:
        message = "source_surfaces :  %s\ntarget_surfaces :  %s\ntarget_type :  %s\nbc_pairs :  %s" % (self._source_surfaces, self._target_surfaces, self._target_type, '[' + ''.join('\n' + str(data) for data in self._bc_pairs) + ']')
        message += ''.join('\n' + str(key) + ' : ' + str(value) for key, value in self._custom_params.items())
        return message

    @property
    def source_surfaces(self) -> Iterable[int]:
        """Ids of source surfaces.
        """
        return self._source_surfaces

    @source_surfaces.setter
    def source_surfaces(self, value: Iterable[int]):
        self._source_surfaces = value

    @property
    def target_surfaces(self) -> Iterable[int]:
        """Ids of target surfaces.
        """
        return self._target_surfaces

    @target_surfaces.setter
    def target_surfaces(self, value: Iterable[int]):
        self._target_surfaces = value

    @property
    def target_type(self) -> MatchPairTargetType:
        """Option to specify target surface type.
        """
        return self._target_type

    @target_type.setter
    def target_type(self, value: MatchPairTargetType):
        self._target_type = value

    @property
    def bc_pairs(self) -> List[BCPair]:
        """Array of boundary condition pairs.
        """
        return self._bc_pairs

    @bc_pairs.setter
    def bc_pairs(self, value: List[BCPair]):
        self._bc_pairs = value



""" Auto-generated file. DO NOT MODIFY """
from __future__ import annotations
from ansys.meshing.prime.internals.comm_manager import CommunicationManager
from ansys.meshing.prime.params.primestructs import *
from ansys.meshing.prime.autogen.coreobject import *
from typing import List, Any, Union

class VolumeSweeper(CoreObject):
    """VolumeSweeper class provide functions to volume mesh a given set of topovolumes by sweeping or stacking a set of face and edge zonelets. Provide operations to generate volume mesh using stacker technology.

    """

    def __init__(self, model: CommunicationManager, part_id: int):
        """ Initialize VolumeSweeper  """
        self._model = model
        self._comm = model._communicator
        command_name = "PrimeMesh::VolumeSweeper/Construct"
        args = {"ModelID" : model._object_id , "PartID" : part_id, "MaxID" : -1}
        result = self._comm.serve(model, command_name, args=args)
        self._object_id = result["ObjectIndex"]
        self._freeze()

    def __enter__(self):
        """ Enter context for VolumeSweeper. """
        return self

    def __exit__(self, type, value, traceback) :
        """ Exit context for VolumeSweeper. """
        command_name = "PrimeMesh::VolumeSweeper/Destruct"
        self._comm.serve(self._model, command_name, self._object_id, args={})

    def create_base_face(self, topo_volume_ids : Iterable[int], params : MeshStackerParams) -> MeshStackerResults:
        """ Creates a face at the specified origin perpendicular to the specified direction. Also, imprint model edges on the face, make necessary edge repairs, and duplicate relevant size controls on the base face.


        Parameters
        ----------
        topo_volume_ids : Iterable[int]
            Ids of volumes that need to be meshed.
        params : MeshStackerParams
            Mesh stacker parameters.

        Returns
        -------
        MeshStackerResults
            Returns the MeshStackerResults.


        Examples
        --------
        >>> results = volumesweeper.create_base_face(topo_volume_ids, params)

        """
        if not isinstance(topo_volume_ids, Iterable):
            raise TypeError("Invalid argument type passed for topo_volume_ids, valid argument type is Iterable[int].")
        if not isinstance(params, MeshStackerParams):
            raise TypeError("Invalid argument type passed for params, valid argument type is MeshStackerParams.")
        args = {"topo_volume_ids" : topo_volume_ids,
        "params" : params._jsonify()}
        command_name = "PrimeMesh::VolumeSweeper/CreateBaseFace"
        self._model._print_logs_before_command("create_base_face", args)
        result = self._comm.serve(self._model, command_name, self._object_id, args=args)
        self._model._print_logs_after_command("create_base_face", MeshStackerResults(model = self._model, json_data = result))
        return MeshStackerResults(model = self._model, json_data = result)

    def stack_base_face(self, base_face_ids : Iterable[int], topo_volume_ids : Iterable[int], params : MeshStackerParams) -> MeshStackerResults:
        """ Generates volume mesh stacking a meshed face layer by layer along the given direction. Calculates the stack layers using size controls and global size parameters.


        Parameters
        ----------
        base_face_ids : Iterable[int]
            Ids of base faces to be stacked
        topo_volume_ids : Iterable[int]
            Ids of volumes that need to be meshed.
        params : MeshStackerParams
            Mesh stacker parameters.

        Returns
        -------
        MeshStackerResults
            Returns the MeshStackerResults.


        Examples
        --------
        >>> results = volumesweeper.stack_base_face(base_face_ids, topo_volume_ids, params)

        """
        if not isinstance(base_face_ids, Iterable):
            raise TypeError("Invalid argument type passed for base_face_ids, valid argument type is Iterable[int].")
        if not isinstance(topo_volume_ids, Iterable):
            raise TypeError("Invalid argument type passed for topo_volume_ids, valid argument type is Iterable[int].")
        if not isinstance(params, MeshStackerParams):
            raise TypeError("Invalid argument type passed for params, valid argument type is MeshStackerParams.")
        args = {"base_face_ids" : base_face_ids,
        "topo_volume_ids" : topo_volume_ids,
        "params" : params._jsonify()}
        command_name = "PrimeMesh::VolumeSweeper/StackBaseFace"
        self._model._print_logs_before_command("stack_base_face", args)
        result = self._comm.serve(self._model, command_name, self._object_id, args=args)
        self._model._print_logs_after_command("stack_base_face", MeshStackerResults(model = self._model, json_data = result))
        return MeshStackerResults(model = self._model, json_data = result)



"""Version of PyPrimeMesh Library."""
# Version
# ------------------------------------------------------------------------------

try:
    import importlib.metadata as importlib_metadata
except ModuleNotFoundError:  # pragma: no cover
    import importlib_metadata  # type: ignore

__version__ = importlib_metadata.version("ansys-meshing-prime")

# ------------------------------------------------------------------------------



"""Module for general utilities of the project."""
import logging
import os
import shutil
import subprocess
from contextlib import contextmanager
from typing import List, Optional

import ansys.meshing.prime.internals.config as config
import ansys.meshing.prime.internals.defaults as defaults

_LOCAL_PORTS = []
_PRIME_CONTAINER_COUNT = 0


def make_unique_container_name(name: str):
    """Make a unique container name.

    Parameters
    ----------
    name : str
        Original name prefix that is provided.

    Returns
    -------
    str
        Unique name with a numeric integer added as suffix.
    """
    global _PRIME_CONTAINER_COUNT
    _PRIME_CONTAINER_COUNT = _PRIME_CONTAINER_COUNT + 1
    return f'{name}-{_PRIME_CONTAINER_COUNT}'


def to_camel_case(snake_str):
    """Transform snake_case string to camelCase.

    Parameters
    ----------
    snake_str : str
        snake_case string.

    Returns
    -------
    str
        String converted to camelCase.
    """
    components = snake_str.split('_')
    # We capitalize the first letter of each component except the first one
    # with the 'title' method and join them together.
    return components[0] + ''.join(x.title() for x in components[1:])


def get_child_processes(process):
    """Get child processes of a process.

    Parameters
    ----------
    process : int
        Process ID of the parent process.

    Returns
    -------
    List
        Process IDs of the processes.
    """
    children = []
    cmd = subprocess.Popen("pgrep -P %d" % process, shell=True, stdout=subprocess.PIPE)
    out = cmd.stdout.read().decode("utf-8")
    cmd.wait()
    for pid in out.split("\n")[:1]:
        if pid.strip() == '':
            break
        ps_cmd = subprocess.Popen(
            "ps -o cmd= {}".format(int(pid)), stdout=subprocess.PIPE, shell=True
        )
        ps_out = ps_cmd.stdout.read().decode("utf-8")
        ps_cmd.wait()
        cmd_name = ps_out.split()[0]
        if "AnsysPrimeServer" in cmd_name:
            children.append(int(pid))
        else:
            children += get_child_processes(int(pid))
    return children


def terminate_process(process: subprocess):
    """Terminates a process.

    Parameters
    ----------
    process : subprocess
        Process to terminate.
    """
    import signal
    import sys

    if sys.platform.startswith('win32'):
        # process.send_signal(signal.CTRL_C_EVENT)
        process.send_signal(signal.CTRL_BREAK_EVENT)
    else:
        for child in get_child_processes(process.pid):
            os.kill(child, signal.SIGTERM)

    if process.stdin is not None:
        process.stdin.close()
    if process.stdout is not None:
        process.stdout.close()
    if process.stderr is not None:
        process.stderr.close()
    process.terminate()
    process.wait()


def print_logs_before_command(logger: logging.Logger, command: str, args):
    """Print logs before running a command.

    Parameters
    ----------
    logger : logging.Logger
        Logger to print to.
    command : str
        Command to run.
    args : str
        Arguments of the command.
    """
    logger.info("Executing " + command)
    if logger.isEnabledFor(logging.DEBUG):
        logger.debug("Command: " + command)
        logger.debug("Args:")
        for key in args:
            logger.debug("    " + key + ":")
            val = args[key]
            printable_str = ""
            if hasattr(val, '__str__'):
                printable_str = val.__str__()
            elif type(val) == 'str':
                printable_str = val
            else:
                printable_str = str(val)
            for line in printable_str.splitlines():
                logger.debug("        " + line)
        logger.debug("")


def print_logs_after_command(logger: logging.Logger, command: str, ret):
    """Print logs after running a command.

    Parameters
    ----------
    logger : logging.Logger
        Logger to print to.
    command : str
        Command to run.
    ret : str
        Return type of the command.
    """
    logger.info("Finished " + command)
    if logger.isEnabledFor(logging.DEBUG):
        logger.debug("Return: ")
        printable_str = ""
        if hasattr(ret, '__str__'):
            printable_str = ret.__str__()
        elif type(ret) == 'str':
            printable_str = ret
        else:
            printable_str = str(ret)
        for line in printable_str.splitlines():
            logger.debug("        " + line)
        logger.debug("")


def print_beta_api_warning(logger: logging.Logger, command: str):
    """Print warning for beta API.

    Parameters
    ----------
    logger : logging.Logger
        Logger to print to.
    command : str
        Command to run.
    """
    if logger.isEnabledFor(logging.WARNING):
        logger.warning(
            f"This API {command} is a Beta. API Behaviorand implementation may change in future."
        )


def launch_prime_github_container(
    mount_host: str = defaults.get_user_data_path(),
    mount_image: str = defaults.get_user_data_path_for_containers(),
    port: int = defaults.port(),
    name: str = "ansys-prime-server",
    version: Optional[str] = None,
):
    """Launch a container.

    Parameters
    ----------
    mount_host : str, optional
        IP address for the container to mount. The default is ``defaults.get_user_data_path()``.
    mount_image : str, optional
        Name of the path to the container to mount. The default is
        ``defaults.get_user_data_path_for_containers()``.
    port : int, optional
        Port to expose. The default is ``defaults.port()``.
    name : str, optional
        Name of the container. The default is ``"ansys-prime-server"``.
    version : str, optional
        Version of the container to retrieve. The default is ``None``.

    Raises
    ------
    ValueError
        License is not available.
    """
    license_file = os.environ.get('ANSYSLMD_LICENSE_FILE', None)
    image_name = os.environ.get('PYPRIMEMESH_IMAGE_NAME', 'ghcr.io/ansys/prime')
    if license_file is None:
        raise ValueError('Licensing information to launch container not found')
    if version is None:
        version = os.environ.get('PYPRIMEMESH_IMAGE_TAG', 'latest')
    docker_command = [
        'docker',
        'run',
        '-d',
        '--rm',
        '--name',
        f'{name}',
        '-p',
        f'{port}:{port}',
        '-v',
        f'{mount_host}:{mount_image}',
        '-e',
        f'ANSYSLMD_LICENSE_FILE={license_file}',
        f'{image_name}:{version}',
        '--port',
        f'{port}',
    ]
    subprocess.run(docker_command, stdout=subprocess.DEVNULL)


def stop_prime_github_container(name):
    """Stop a running container.

    Parameters
    ----------
    name : str
        Name of the container.
    """
    subprocess.run(['docker', 'stop', f'{name}'], stdout=subprocess.DEVNULL)


@contextmanager
def file_read_context(model, file_name: str):
    """Upload context.

    Upload context to a model.

    Parameters
    ----------
    model : Model
        Model to upload the context to.
    file_name : str
        Name of the file containing the context.

    Yields
    ------
    str
        File name of the context.
    """
    if config.file_existence_check_enabled() and not os.path.exists(file_name):
        raise FileNotFoundError(f'Given file name "{file_name}" is not found on local disk')
    if config.using_container():
        base_file_name = os.path.basename(file_name)
        temp_file_name = os.path.join(defaults.get_examples_path(), base_file_name)
        is_copy: bool = file_name != temp_file_name
        if is_copy:
            shutil.copyfile(file_name, temp_file_name)
        container_file_name = os.path.join(
            defaults.get_examples_path_for_containers(), base_file_name
        )
        container_file_name = container_file_name.replace(os.path.sep, '/')
        yield container_file_name
        if is_copy:
            os.remove(temp_file_name)
    elif config.has_pim():
        temp_file_name = os.path.basename(file_name)
        model.file_service.upload_file(file_name)
        yield temp_file_name
    else:
        yield file_name


def port_in_use(port, host=defaults.ip()):
    """Check if a port is in use on a given host.

    This method must actually check is is a "bind" of the port to the
    address. Just checking if a socket can be created is insufficient
    because it's possible to run into permission errors like this one:

    - An attempt was made to access a socket in a way forbidden by its
      access permissions.

    Parameters
    ----------
    port : int
        Port to check.
    host : str, optional
        IP address to check. The default is ``defaults.ip()``.

    Returns
    -------
    bool
        ``True`` if the port is in use, ``False`` if the port is available.
    """
    import socket

    with socket.socket(socket.AF_INET, socket.SOCK_STREAM) as sock:
        try:
            sock.bind((host, port))
            return False
        except:
            return True


def get_available_local_port(init_port: int = defaults.port()):
    """Get which ports are available and return the first one.

    Parameters
    ----------
    init_port : int, optional
        Port to start the search from. The default is ``defaults.port()```.

    Returns
    -------
    int
        First available port.
    """
    port = init_port
    while port_in_use(port) or port in _LOCAL_PORTS:
        port += 1
    _LOCAL_PORTS.append(port)
    return port


@contextmanager
def file_read_context_list(model, file_names: List[str]):
    """Upload context.

    Upload context to a model.

    Parameters
    ----------
    model : Model
        Model to upload context to.
    file_names : List[str]
        List of files with the context.

    Yields
    ------
    List[str]
        List of context files.
    """
    if config.file_existence_check_enabled():
        for file in file_names:
            if not os.path.exists(file):
                error_msg = f"File {file} given for read is missing from local disk."
                raise FileNotFoundError(error_msg)
    if config.using_container():
        base_names = [os.path.basename(file) for file in file_names]
        temp_names = [os.path.join(defaults.get_examples_path(), base) for base in base_names]
        for file, temp in zip(file_names, temp_names):
            shutil.copyfile(file, temp)
        container_files = [
            os.path.join(defaults.get_examples_path_for_containers(), base) for base in base_names
        ]
        container_files = [file.replace(os.path.sep, '/') for file in container_files]
        yield container_files
        for temp_file in temp_names:
            os.remove(temp_file)
    elif config.has_pim():
        temp_files = [os.path.basename(file) for file in file_names]
        for file in file_names:
            model.file_service.upload_file(file)
        yield temp_files
    else:
        yield file_names


@contextmanager
def file_write_context(model, file_name: str):
    """Download context.

    Download context from a model and write it to a local file.

    Parameters
    ----------
    model : Model
        Model to download context from.
    file_name : str
        Name of the file to write the context to.

    Yields
    ------
    str
        Name of the file to which context has been written.
    """
    if config.using_container():
        base_file_name = os.path.basename(file_name)
        temp_file_name = os.path.join(defaults.get_output_path_for_containers(), base_file_name)
        temp_file_name = temp_file_name.replace(os.path.sep, '/')
        if not os.path.exists(defaults.get_output_path()):
            os.makedirs(defaults.get_output_path())
        yield temp_file_name
        # Copy temp_file_name to directory which was asked
        local_file_name = os.path.join(defaults.get_output_path(), base_file_name)
        shutil.copyfile(local_file_name, file_name)
        os.remove(local_file_name)
    elif config.has_pim():
        temp_file_name = os.path.basename(file_name)
        file_dir = os.path.dirname(file_name)
        yield temp_file_name
        model.file_service.download_file(temp_file_name, file_dir)
    else:
        yield file_name



""" Auto-generated file. DO NOT MODIFY """
import enum
from typing import Dict, Any, Union, List, Iterable
from ansys.meshing.prime.internals.comm_manager import CommunicationManager
from ansys.meshing.prime.internals import utils
from ansys.meshing.prime.autogen.coreobject import *
import numpy as np

from ansys.meshing.prime.params.primestructs import *

class IntersectionMask(enum.IntEnum):
    """Scaffold parameters use intersection mask to define nature of intersection computation.
    """
    EDGEEDGE = 1
    """Performs edge to edge intersection."""
    FACEFACE = 2
    """Performs face to face intersection."""
    FACEFACEANDEDGEEDGE = 3
    """Perform face to face and edge to edge intersections."""

class ScaffolderParams(CoreObject):
    """Parameters to control scaffold operation.
    """
    _default_params = {}

    def __initialize(
            self,
            absolute_dist_tol: float,
            intersection_control_mask: IntersectionMask,
            constant_mesh_size: float):
        self._absolute_dist_tol = absolute_dist_tol
        self._intersection_control_mask = IntersectionMask(intersection_control_mask)
        self._constant_mesh_size = constant_mesh_size

    def __init__(
            self,
            model: CommunicationManager=None,
            absolute_dist_tol: float = None,
            intersection_control_mask: IntersectionMask = None,
            constant_mesh_size: float = None,
            json_data : dict = None,
             **kwargs):
        """Initializes the ScaffolderParams.

        Parameters
        ----------
        model: Model
            Model to create a ScaffolderParams object with default parameters.
        absolute_dist_tol: float, optional
            Defines the maximum gap to connect.
        intersection_control_mask: IntersectionMask, optional
            Specifies the nature of intersection to be computed.
        constant_mesh_size: float, optional
            Defines the constant edge mesh size to check connection.
        json_data: dict, optional
            JSON dictionary to create a ScaffolderParams object with provided parameters.

        Examples
        --------
        >>> scaffolder_params = prime.ScaffolderParams(model = model)
        """
        if json_data:
            self.__initialize(
                json_data["absoluteDistTol"] if "absoluteDistTol" in json_data else None,
                IntersectionMask(json_data["intersectionControlMask"] if "intersectionControlMask" in json_data else None),
                json_data["constantMeshSize"] if "constantMeshSize" in json_data else None)
        else:
            all_field_specified = all(arg is not None for arg in [absolute_dist_tol, intersection_control_mask, constant_mesh_size])
            if all_field_specified:
                self.__initialize(
                    absolute_dist_tol,
                    intersection_control_mask,
                    constant_mesh_size)
            else:
                if model is None:
                    raise ValueError("Invalid assignment. Either pass model or specify all properties")
                else:
                    param_json = model._communicator.initialize_params(model, "ScaffolderParams")
                    json_data = param_json["ScaffolderParams"] if "ScaffolderParams" in param_json else {}
                    self.__initialize(
                        absolute_dist_tol if absolute_dist_tol is not None else ( ScaffolderParams._default_params["absolute_dist_tol"] if "absolute_dist_tol" in ScaffolderParams._default_params else (json_data["absoluteDistTol"] if "absoluteDistTol" in json_data else None)),
                        intersection_control_mask if intersection_control_mask is not None else ( ScaffolderParams._default_params["intersection_control_mask"] if "intersection_control_mask" in ScaffolderParams._default_params else IntersectionMask(json_data["intersectionControlMask"] if "intersectionControlMask" in json_data else None)),
                        constant_mesh_size if constant_mesh_size is not None else ( ScaffolderParams._default_params["constant_mesh_size"] if "constant_mesh_size" in ScaffolderParams._default_params else (json_data["constantMeshSize"] if "constantMeshSize" in json_data else None)))
        self._custom_params = kwargs
        if model is not None:
            [ model._logger.warning(f'Unsupported argument : {key}') for key in kwargs ]
        [setattr(type(self), key, property(lambda self, key = key:  self._custom_params[key] if key in self._custom_params else None,
        lambda self, value, key = key : self._custom_params.update({ key: value }))) for key in kwargs]
        self._freeze()

    @staticmethod
    def set_default(
            absolute_dist_tol: float = None,
            intersection_control_mask: IntersectionMask = None,
            constant_mesh_size: float = None):
        """Set the default values of ScaffolderParams.

        Parameters
        ----------
        absolute_dist_tol: float, optional
            Defines the maximum gap to connect.
        intersection_control_mask: IntersectionMask, optional
            Specifies the nature of intersection to be computed.
        constant_mesh_size: float, optional
            Defines the constant edge mesh size to check connection.
        """
        args = locals()
        [ScaffolderParams._default_params.update({ key: value }) for key, value in args.items() if value is not None]

    @staticmethod
    def print_default():
        """Print the default values of ScaffolderParams.

        Examples
        --------
        >>> ScaffolderParams.print_default()
        """
        message = ""
        message += ''.join(str(key) + ' : ' + str(value) + '\n' for key, value in ScaffolderParams._default_params.items())
        print(message)

    def _jsonify(self) -> Dict[str, Any]:
        json_data = {}
        if self._absolute_dist_tol is not None:
            json_data["absoluteDistTol"] = self._absolute_dist_tol
        if self._intersection_control_mask is not None:
            json_data["intersectionControlMask"] = self._intersection_control_mask
        if self._constant_mesh_size is not None:
            json_data["constantMeshSize"] = self._constant_mesh_size
        [ json_data.update({ utils.to_camel_case(key) : value }) for key, value in self._custom_params.items()]
        return json_data

    def __str__(self) -> str:
        message = "absolute_dist_tol :  %s\nintersection_control_mask :  %s\nconstant_mesh_size :  %s" % (self._absolute_dist_tol, self._intersection_control_mask, self._constant_mesh_size)
        message += ''.join('\n' + str(key) + ' : ' + str(value) for key, value in self._custom_params.items())
        return message

    @property
    def absolute_dist_tol(self) -> float:
        """Defines the maximum gap to connect.
        """
        return self._absolute_dist_tol

    @absolute_dist_tol.setter
    def absolute_dist_tol(self, value: float):
        self._absolute_dist_tol = value

    @property
    def intersection_control_mask(self) -> IntersectionMask:
        """Specifies the nature of intersection to be computed.
        """
        return self._intersection_control_mask

    @intersection_control_mask.setter
    def intersection_control_mask(self, value: IntersectionMask):
        self._intersection_control_mask = value

    @property
    def constant_mesh_size(self) -> float:
        """Defines the constant edge mesh size to check connection.
        """
        return self._constant_mesh_size

    @constant_mesh_size.setter
    def constant_mesh_size(self, value: float):
        self._constant_mesh_size = value

class VolumetricScaffolderParams(CoreObject):
    """Parameters to control delete shadowed topofaces operation.
    """
    _default_params = {}

    def __initialize(
            self,
            absolute_dist_tol: float,
            only_check_exact_overlaps: bool):
        self._absolute_dist_tol = absolute_dist_tol
        self._only_check_exact_overlaps = only_check_exact_overlaps

    def __init__(
            self,
            model: CommunicationManager=None,
            absolute_dist_tol: float = None,
            only_check_exact_overlaps: bool = None,
            json_data : dict = None,
             **kwargs):
        """Initializes the VolumetricScaffolderParams.

        Parameters
        ----------
        model: Model
            Model to create a VolumetricScaffolderParams object with default parameters.
        absolute_dist_tol: float, optional
            Specify distance tolerance between overlapping faces.
        only_check_exact_overlaps: bool, optional
            Check only for fully overlapping topofaces when true.
        json_data: dict, optional
            JSON dictionary to create a VolumetricScaffolderParams object with provided parameters.

        Examples
        --------
        >>> volumetric_scaffolder_params = prime.VolumetricScaffolderParams(model = model)
        """
        if json_data:
            self.__initialize(
                json_data["absoluteDistTol"] if "absoluteDistTol" in json_data else None,
                json_data["onlyCheckExactOverlaps"] if "onlyCheckExactOverlaps" in json_data else None)
        else:
            all_field_specified = all(arg is not None for arg in [absolute_dist_tol, only_check_exact_overlaps])
            if all_field_specified:
                self.__initialize(
                    absolute_dist_tol,
                    only_check_exact_overlaps)
            else:
                if model is None:
                    raise ValueError("Invalid assignment. Either pass model or specify all properties")
                else:
                    param_json = model._communicator.initialize_params(model, "VolumetricScaffolderParams")
                    json_data = param_json["VolumetricScaffolderParams"] if "VolumetricScaffolderParams" in param_json else {}
                    self.__initialize(
                        absolute_dist_tol if absolute_dist_tol is not None else ( VolumetricScaffolderParams._default_params["absolute_dist_tol"] if "absolute_dist_tol" in VolumetricScaffolderParams._default_params else (json_data["absoluteDistTol"] if "absoluteDistTol" in json_data else None)),
                        only_check_exact_overlaps if only_check_exact_overlaps is not None else ( VolumetricScaffolderParams._default_params["only_check_exact_overlaps"] if "only_check_exact_overlaps" in VolumetricScaffolderParams._default_params else (json_data["onlyCheckExactOverlaps"] if "onlyCheckExactOverlaps" in json_data else None)))
        self._custom_params = kwargs
        if model is not None:
            [ model._logger.warning(f'Unsupported argument : {key}') for key in kwargs ]
        [setattr(type(self), key, property(lambda self, key = key:  self._custom_params[key] if key in self._custom_params else None,
        lambda self, value, key = key : self._custom_params.update({ key: value }))) for key in kwargs]
        self._freeze()

    @staticmethod
    def set_default(
            absolute_dist_tol: float = None,
            only_check_exact_overlaps: bool = None):
        """Set the default values of VolumetricScaffolderParams.

        Parameters
        ----------
        absolute_dist_tol: float, optional
            Specify distance tolerance between overlapping faces.
        only_check_exact_overlaps: bool, optional
            Check only for fully overlapping topofaces when true.
        """
        args = locals()
        [VolumetricScaffolderParams._default_params.update({ key: value }) for key, value in args.items() if value is not None]

    @staticmethod
    def print_default():
        """Print the default values of VolumetricScaffolderParams.

        Examples
        --------
        >>> VolumetricScaffolderParams.print_default()
        """
        message = ""
        message += ''.join(str(key) + ' : ' + str(value) + '\n' for key, value in VolumetricScaffolderParams._default_params.items())
        print(message)

    def _jsonify(self) -> Dict[str, Any]:
        json_data = {}
        if self._absolute_dist_tol is not None:
            json_data["absoluteDistTol"] = self._absolute_dist_tol
        if self._only_check_exact_overlaps is not None:
            json_data["onlyCheckExactOverlaps"] = self._only_check_exact_overlaps
        [ json_data.update({ utils.to_camel_case(key) : value }) for key, value in self._custom_params.items()]
        return json_data

    def __str__(self) -> str:
        message = "absolute_dist_tol :  %s\nonly_check_exact_overlaps :  %s" % (self._absolute_dist_tol, self._only_check_exact_overlaps)
        message += ''.join('\n' + str(key) + ' : ' + str(value) for key, value in self._custom_params.items())
        return message

    @property
    def absolute_dist_tol(self) -> float:
        """Specify distance tolerance between overlapping faces.
        """
        return self._absolute_dist_tol

    @absolute_dist_tol.setter
    def absolute_dist_tol(self, value: float):
        self._absolute_dist_tol = value

    @property
    def only_check_exact_overlaps(self) -> bool:
        """Check only for fully overlapping topofaces when true.
        """
        return self._only_check_exact_overlaps

    @only_check_exact_overlaps.setter
    def only_check_exact_overlaps(self, value: bool):
        self._only_check_exact_overlaps = value

class ScaffolderResults(CoreObject):
    """Results structure associated to scaffold operation.
    """
    _default_params = {}

    def __initialize(
            self,
            n_incomplete_topo_faces: int,
            error_code: ErrorCode):
        self._n_incomplete_topo_faces = n_incomplete_topo_faces
        self._error_code = ErrorCode(error_code)

    def __init__(
            self,
            model: CommunicationManager=None,
            n_incomplete_topo_faces: int = None,
            error_code: ErrorCode = None,
            json_data : dict = None,
             **kwargs):
        """Initializes the ScaffolderResults.

        Parameters
        ----------
        model: Model
            Model to create a ScaffolderResults object with default parameters.
        n_incomplete_topo_faces: int, optional
            Number of topofaces failed in scaffold operation.
        error_code: ErrorCode, optional
            Error code associated with scaffold operation.
        json_data: dict, optional
            JSON dictionary to create a ScaffolderResults object with provided parameters.

        Examples
        --------
        >>> scaffolder_results = prime.ScaffolderResults(model = model)
        """
        if json_data:
            self.__initialize(
                json_data["nIncompleteTopoFaces"] if "nIncompleteTopoFaces" in json_data else None,
                ErrorCode(json_data["errorCode"] if "errorCode" in json_data else None))
        else:
            all_field_specified = all(arg is not None for arg in [n_incomplete_topo_faces, error_code])
            if all_field_specified:
                self.__initialize(
                    n_incomplete_topo_faces,
                    error_code)
            else:
                if model is None:
                    raise ValueError("Invalid assignment. Either pass model or specify all properties")
                else:
                    param_json = model._communicator.initialize_params(model, "ScaffolderResults")
                    json_data = param_json["ScaffolderResults"] if "ScaffolderResults" in param_json else {}
                    self.__initialize(
                        n_incomplete_topo_faces if n_incomplete_topo_faces is not None else ( ScaffolderResults._default_params["n_incomplete_topo_faces"] if "n_incomplete_topo_faces" in ScaffolderResults._default_params else (json_data["nIncompleteTopoFaces"] if "nIncompleteTopoFaces" in json_data else None)),
                        error_code if error_code is not None else ( ScaffolderResults._default_params["error_code"] if "error_code" in ScaffolderResults._default_params else ErrorCode(json_data["errorCode"] if "errorCode" in json_data else None)))
        self._custom_params = kwargs
        if model is not None:
            [ model._logger.warning(f'Unsupported argument : {key}') for key in kwargs ]
        [setattr(type(self), key, property(lambda self, key = key:  self._custom_params[key] if key in self._custom_params else None,
        lambda self, value, key = key : self._custom_params.update({ key: value }))) for key in kwargs]
        self._freeze()

    @staticmethod
    def set_default(
            n_incomplete_topo_faces: int = None,
            error_code: ErrorCode = None):
        """Set the default values of ScaffolderResults.

        Parameters
        ----------
        n_incomplete_topo_faces: int, optional
            Number of topofaces failed in scaffold operation.
        error_code: ErrorCode, optional
            Error code associated with scaffold operation.
        """
        args = locals()
        [ScaffolderResults._default_params.update({ key: value }) for key, value in args.items() if value is not None]

    @staticmethod
    def print_default():
        """Print the default values of ScaffolderResults.

        Examples
        --------
        >>> ScaffolderResults.print_default()
        """
        message = ""
        message += ''.join(str(key) + ' : ' + str(value) + '\n' for key, value in ScaffolderResults._default_params.items())
        print(message)

    def _jsonify(self) -> Dict[str, Any]:
        json_data = {}
        if self._n_incomplete_topo_faces is not None:
            json_data["nIncompleteTopoFaces"] = self._n_incomplete_topo_faces
        if self._error_code is not None:
            json_data["errorCode"] = self._error_code
        [ json_data.update({ utils.to_camel_case(key) : value }) for key, value in self._custom_params.items()]
        return json_data

    def __str__(self) -> str:
        message = "n_incomplete_topo_faces :  %s\nerror_code :  %s" % (self._n_incomplete_topo_faces, self._error_code)
        message += ''.join('\n' + str(key) + ' : ' + str(value) for key, value in self._custom_params.items())
        return message

    @property
    def n_incomplete_topo_faces(self) -> int:
        """Number of topofaces failed in scaffold operation.
        """
        return self._n_incomplete_topo_faces

    @n_incomplete_topo_faces.setter
    def n_incomplete_topo_faces(self, value: int):
        self._n_incomplete_topo_faces = value

    @property
    def error_code(self) -> ErrorCode:
        """Error code associated with scaffold operation.
        """
        return self._error_code

    @error_code.setter
    def error_code(self, value: ErrorCode):
        self._error_code = value

class ScaffolderSplitResults(CoreObject):
    """Result structure associated to split topofaces operation.
    """
    _default_params = {}

    def __initialize(
            self,
            new_faces: Iterable[int],
            error_code: ErrorCode):
        self._new_faces = new_faces if isinstance(new_faces, np.ndarray) else np.array(new_faces, dtype=np.int32) if new_faces is not None else None
        self._error_code = ErrorCode(error_code)

    def __init__(
            self,
            model: CommunicationManager=None,
            new_faces: Iterable[int] = None,
            error_code: ErrorCode = None,
            json_data : dict = None,
             **kwargs):
        """Initializes the ScaffolderSplitResults.

        Parameters
        ----------
        model: Model
            Model to create a ScaffolderSplitResults object with default parameters.
        new_faces: Iterable[int], optional
            Topofaces created after split operation.
        error_code: ErrorCode, optional
            Error code associated with split topofaces operation.
        json_data: dict, optional
            JSON dictionary to create a ScaffolderSplitResults object with provided parameters.

        Examples
        --------
        >>> scaffolder_split_results = prime.ScaffolderSplitResults(model = model)
        """
        if json_data:
            self.__initialize(
                json_data["newFaces"] if "newFaces" in json_data else None,
                ErrorCode(json_data["errorCode"] if "errorCode" in json_data else None))
        else:
            all_field_specified = all(arg is not None for arg in [new_faces, error_code])
            if all_field_specified:
                self.__initialize(
                    new_faces,
                    error_code)
            else:
                if model is None:
                    raise ValueError("Invalid assignment. Either pass model or specify all properties")
                else:
                    param_json = model._communicator.initialize_params(model, "ScaffolderSplitResults")
                    json_data = param_json["ScaffolderSplitResults"] if "ScaffolderSplitResults" in param_json else {}
                    self.__initialize(
                        new_faces if new_faces is not None else ( ScaffolderSplitResults._default_params["new_faces"] if "new_faces" in ScaffolderSplitResults._default_params else (json_data["newFaces"] if "newFaces" in json_data else None)),
                        error_code if error_code is not None else ( ScaffolderSplitResults._default_params["error_code"] if "error_code" in ScaffolderSplitResults._default_params else ErrorCode(json_data["errorCode"] if "errorCode" in json_data else None)))
        self._custom_params = kwargs
        if model is not None:
            [ model._logger.warning(f'Unsupported argument : {key}') for key in kwargs ]
        [setattr(type(self), key, property(lambda self, key = key:  self._custom_params[key] if key in self._custom_params else None,
        lambda self, value, key = key : self._custom_params.update({ key: value }))) for key in kwargs]
        self._freeze()

    @staticmethod
    def set_default(
            new_faces: Iterable[int] = None,
            error_code: ErrorCode = None):
        """Set the default values of ScaffolderSplitResults.

        Parameters
        ----------
        new_faces: Iterable[int], optional
            Topofaces created after split operation.
        error_code: ErrorCode, optional
            Error code associated with split topofaces operation.
        """
        args = locals()
        [ScaffolderSplitResults._default_params.update({ key: value }) for key, value in args.items() if value is not None]

    @staticmethod
    def print_default():
        """Print the default values of ScaffolderSplitResults.

        Examples
        --------
        >>> ScaffolderSplitResults.print_default()
        """
        message = ""
        message += ''.join(str(key) + ' : ' + str(value) + '\n' for key, value in ScaffolderSplitResults._default_params.items())
        print(message)

    def _jsonify(self) -> Dict[str, Any]:
        json_data = {}
        if self._new_faces is not None:
            json_data["newFaces"] = self._new_faces
        if self._error_code is not None:
            json_data["errorCode"] = self._error_code
        [ json_data.update({ utils.to_camel_case(key) : value }) for key, value in self._custom_params.items()]
        return json_data

    def __str__(self) -> str:
        message = "new_faces :  %s\nerror_code :  %s" % (self._new_faces, self._error_code)
        message += ''.join('\n' + str(key) + ' : ' + str(value) for key, value in self._custom_params.items())
        return message

    @property
    def new_faces(self) -> Iterable[int]:
        """Topofaces created after split operation.
        """
        return self._new_faces

    @new_faces.setter
    def new_faces(self, value: Iterable[int]):
        self._new_faces = value

    @property
    def error_code(self) -> ErrorCode:
        """Error code associated with split topofaces operation.
        """
        return self._error_code

    @error_code.setter
    def error_code(self, value: ErrorCode):
        self._error_code = value

class ScaffolderMergeResults(CoreObject):
    """Result structure associated with merge overlapping topofaces and delete shadowed topofaces operations.
    """
    _default_params = {}

    def __initialize(
            self,
            n_merged: int,
            error_code: ErrorCode):
        self._n_merged = n_merged
        self._error_code = ErrorCode(error_code)

    def __init__(
            self,
            model: CommunicationManager=None,
            n_merged: int = None,
            error_code: ErrorCode = None,
            json_data : dict = None,
             **kwargs):
        """Initializes the ScaffolderMergeResults.

        Parameters
        ----------
        model: Model
            Model to create a ScaffolderMergeResults object with default parameters.
        n_merged: int, optional
            Number of merged topofaces.
        error_code: ErrorCode, optional
            Error code associated with merge overlapping topofaces operation.
        json_data: dict, optional
            JSON dictionary to create a ScaffolderMergeResults object with provided parameters.

        Examples
        --------
        >>> scaffolder_merge_results = prime.ScaffolderMergeResults(model = model)
        """
        if json_data:
            self.__initialize(
                json_data["nMerged"] if "nMerged" in json_data else None,
                ErrorCode(json_data["errorCode"] if "errorCode" in json_data else None))
        else:
            all_field_specified = all(arg is not None for arg in [n_merged, error_code])
            if all_field_specified:
                self.__initialize(
                    n_merged,
                    error_code)
            else:
                if model is None:
                    raise ValueError("Invalid assignment. Either pass model or specify all properties")
                else:
                    param_json = model._communicator.initialize_params(model, "ScaffolderMergeResults")
                    json_data = param_json["ScaffolderMergeResults"] if "ScaffolderMergeResults" in param_json else {}
                    self.__initialize(
                        n_merged if n_merged is not None else ( ScaffolderMergeResults._default_params["n_merged"] if "n_merged" in ScaffolderMergeResults._default_params else (json_data["nMerged"] if "nMerged" in json_data else None)),
                        error_code if error_code is not None else ( ScaffolderMergeResults._default_params["error_code"] if "error_code" in ScaffolderMergeResults._default_params else ErrorCode(json_data["errorCode"] if "errorCode" in json_data else None)))
        self._custom_params = kwargs
        if model is not None:
            [ model._logger.warning(f'Unsupported argument : {key}') for key in kwargs ]
        [setattr(type(self), key, property(lambda self, key = key:  self._custom_params[key] if key in self._custom_params else None,
        lambda self, value, key = key : self._custom_params.update({ key: value }))) for key in kwargs]
        self._freeze()

    @staticmethod
    def set_default(
            n_merged: int = None,
            error_code: ErrorCode = None):
        """Set the default values of ScaffolderMergeResults.

        Parameters
        ----------
        n_merged: int, optional
            Number of merged topofaces.
        error_code: ErrorCode, optional
            Error code associated with merge overlapping topofaces operation.
        """
        args = locals()
        [ScaffolderMergeResults._default_params.update({ key: value }) for key, value in args.items() if value is not None]

    @staticmethod
    def print_default():
        """Print the default values of ScaffolderMergeResults.

        Examples
        --------
        >>> ScaffolderMergeResults.print_default()
        """
        message = ""
        message += ''.join(str(key) + ' : ' + str(value) + '\n' for key, value in ScaffolderMergeResults._default_params.items())
        print(message)

    def _jsonify(self) -> Dict[str, Any]:
        json_data = {}
        if self._n_merged is not None:
            json_data["nMerged"] = self._n_merged
        if self._error_code is not None:
            json_data["errorCode"] = self._error_code
        [ json_data.update({ utils.to_camel_case(key) : value }) for key, value in self._custom_params.items()]
        return json_data

    def __str__(self) -> str:
        message = "n_merged :  %s\nerror_code :  %s" % (self._n_merged, self._error_code)
        message += ''.join('\n' + str(key) + ' : ' + str(value) for key, value in self._custom_params.items())
        return message

    @property
    def n_merged(self) -> int:
        """Number of merged topofaces.
        """
        return self._n_merged

    @n_merged.setter
    def n_merged(self, value: int):
        self._n_merged = value

    @property
    def error_code(self) -> ErrorCode:
        """Error code associated with merge overlapping topofaces operation.
        """
        return self._error_code

    @error_code.setter
    def error_code(self, value: ErrorCode):
        self._error_code = value
